import json
import os
import textwrap

from conan.test.assets.genconanfile import GenConanfile
from conan.test.utils.tools import TestClient


def test_upload_pkg_list():
    """ Test how a custom command can create an upload pkglist and print it
    """
    c = TestClient(default_server_user=True, light=True)
    mycommand = textwrap.dedent("""
        import json
        import os

        from conan.cli.command import conan_command, OnceArgument
        from conan.api.model import ListPattern
        from conan.api.output import cli_out_write

        @conan_command(group="custom commands")
        def upload_pkglist(conan_api, parser, *args, **kwargs):
            \"""
            create an upload pkglist
            \"""
            parser.add_argument('reference',
                                help="Recipe reference or package reference, can contain * as "
                                      "wildcard at any reference field.")
            # using required, we may want to pass this as a positional argument?
            parser.add_argument("-r", "--remote", action=OnceArgument, required=True,
                                help='Upload to this specific remote')
            args = parser.parse_args(*args)

            remote = conan_api.remotes.get(args.remote)
            enabled_remotes = conan_api.remotes.list()

            ref_pattern = ListPattern(args.reference, package_id="*")
            package_list = conan_api.list.select(ref_pattern)
            if not package_list:
                raise ConanException("No recipes found matching pattern '{}'".format(args.reference))

            # Check if the recipes/packages are in the remote
            conan_api.upload.check_upstream(package_list, remote, enabled_remotes)
            conan_api.upload.prepare(package_list, enabled_remotes)
            cli_out_write(json.dumps(package_list.serialize(), indent=4))
        """)

    command_file_path = os.path.join(c.cache_folder, 'extensions',
                                     'commands', 'cmd_upload_pkglist.py')
    c.save({command_file_path: mycommand})
    c.save({"conanfile.py": GenConanfile("pkg", "0.1")})
    c.run("create .")
    c.run('upload-pkglist "*" -r=default', redirect_stdout="mypkglist.json")
    pkglist = c.load("mypkglist.json")
    pkglist = json.loads(pkglist)
    assert pkglist["pkg/0.1"]["revisions"]["485dad6cb11e2fa99d9afbe44a57a164"]["upload"] is True



import os

import pytest

from conan import ConanFile
from conan.tools.microsoft import MSBuild, MSBuildToolchain, is_msvc, is_msvc_static_runtime
from conan.internal.model.conf import ConfDefinition, Conf
from conan.internal.model.settings import Settings
from conan.test.utils.mocks import MockSettings, ConanFileMock, MockOptions
from conan.test.utils.test_files import temp_folder
from conan.internal.util.files import load


def test_msbuild_targets():
    c = ConfDefinition()
    settings = MockSettings({"build_type": "Release",
                             "compiler": "gcc",
                             "compiler.version": "7",
                             "os": "Linux",
                             "arch": "x86_64"})
    conanfile = ConanFileMock()
    conanfile.settings = settings
    conanfile.conf = c.get_conanfile_conf(None)

    msbuild = MSBuild(conanfile)
    cmd = msbuild.command('project.sln', targets=["static", "shared"])

    assert '-target:"static;shared"' in cmd


def test_msbuild_cpu_count():
    c = ConfDefinition()
    c.loads("tools.microsoft.msbuild:max_cpu_count=23")

    settings = MockSettings({"build_type": "Release",
                             "arch": "x86_64"})
    conanfile = ConanFileMock()
    conanfile.settings = settings
    conanfile.conf = c.get_conanfile_conf(None)

    msbuild = MSBuild(conanfile)
    cmd = msbuild.command('project.sln')
    assert 'msbuild.exe "project.sln" -p:Configuration="Release" -p:Platform="x64" -m:"23"' == cmd

    c.loads("tools.microsoft.msbuild:max_cpu_count=0")
    conanfile.conf = c.get_conanfile_conf(None)
    cmd = msbuild.command('project.sln')
    assert 'msbuild.exe "project.sln" -p:Configuration="Release" -p:Platform="x64" -m' == cmd


def test_msbuild_toolset():
    settings = Settings({"build_type": ["Release"],
                         "compiler": {"msvc": {"version": ["193"], "toolset": [None, "v142_xp"]}},
                         "os": ["Windows"],
                         "arch": ["x86_64"]})
    conanfile = ConanFile(None)
    conanfile.conf = Conf()
    conanfile.settings = "os", "compiler", "build_type", "arch"
    conanfile.settings = settings
    conanfile.settings.build_type = "Release"
    conanfile.settings.compiler = "msvc"
    conanfile.settings.compiler.version = "193"
    conanfile.settings.os = "Windows"
    conanfile.settings.arch = "x86_64"

    msbuild = MSBuildToolchain(conanfile)
    assert 'v143' in msbuild.toolset

    conanfile.settings.compiler.toolset = "v142_xp"
    msbuild = MSBuildToolchain(conanfile)
    assert 'v142_xp' in msbuild.toolset


@pytest.mark.parametrize("mode,expected_toolset", [
    ("icx", "Intel C++ Compiler 2021"),
    ("dpcpp", "Intel(R) oneAPI DPC++ Compiler"),
    ("classic", "Intel C++ Compiler 19.2")
])
def test_msbuild_toolset_for_intel_cc(mode, expected_toolset):
    conanfile = ConanFile()
    conanfile.conf = Conf()
    conanfile.settings = "os", "compiler", "build_type", "arch"
    conanfile.settings = Settings({"build_type": ["Release"],
                                   "compiler": {"intel-cc": {"version": ["2021.3"], "mode": [mode]},
                                                "msvc": {"version": ["193"], "cppstd": ["20"]}},
                                   "os": ["Windows"],
                                   "arch": ["x86_64"]})
    conanfile.settings.build_type = "Release"
    conanfile.settings.compiler = "intel-cc"
    conanfile.settings.compiler.version = "2021.3"
    conanfile.settings.compiler.mode = mode
    conanfile.settings.os = "Windows"
    conanfile.settings.arch = "x86_64"

    msbuild = MSBuildToolchain(conanfile)
    assert expected_toolset == msbuild.toolset


def test_msbuild_standard():
    test_folder = temp_folder()
    conanfile = ConanFile()
    conanfile.folders.set_base_generators(test_folder)
    conanfile.conf = Conf()
    conanfile.conf.define("tools.microsoft.msbuild:installation_path", ".")
    conanfile.settings = "os", "compiler", "build_type", "arch"
    conanfile.settings = Settings({"build_type": ["Release"],
                                   "compiler": {"msvc": {"version": ["193"], "cppstd": ["20"],
                                                         "cstd": ["17"]}},
                                   "os": ["Windows"],
                                   "arch": ["x86_64"]})
    conanfile.settings_build = conanfile.settings
    conanfile.settings.build_type = "Release"
    conanfile.settings.compiler = "msvc"
    conanfile.settings.compiler.version = "193"
    conanfile.settings.compiler.cppstd = "20"
    conanfile.settings.compiler.cstd = "17"
    conanfile.settings.os = "Windows"
    conanfile.settings.arch = "x86_64"

    msbuild = MSBuildToolchain(conanfile)
    props_file = os.path.join(test_folder, 'conantoolchain_release_x64.props')
    msbuild.generate()
    assert '<LanguageStandard>stdcpp20</LanguageStandard>' in load(props_file)
    assert '<LanguageStandard_C>stdc17</LanguageStandard_C>' in load(props_file)


def test_resource_compile():
    test_folder = temp_folder()

    settings = Settings({"build_type": ["Release"],
                         "compiler": {"msvc": {"version": ["193"], "cppstd": ["20"]}},
                         "os": ["Windows"],
                         "arch": ["x86_64"]})
    conanfile = ConanFile()
    conanfile.folders.set_base_generators(test_folder)
    conanfile.conf = Conf()
    conanfile.conf.define("tools.microsoft.msbuild:installation_path", ".")
    conanfile.settings = "os", "compiler", "build_type", "arch"
    conanfile.settings = settings
    conanfile.settings_build = settings
    conanfile.settings.build_type = "Release"
    conanfile.settings.compiler = "msvc"
    conanfile.settings.compiler.version = "193"
    conanfile.settings.compiler.cppstd = "20"
    conanfile.settings.os = "Windows"
    conanfile.settings.arch = "x86_64"

    msbuild = MSBuildToolchain(conanfile)
    msbuild.preprocessor_definitions["MYTEST"] = "MYVALUE"
    props_file = os.path.join(test_folder, 'conantoolchain_release_x64.props')
    msbuild.generate()
    expected = """
        <ResourceCompile>
          <PreprocessorDefinitions>
             MYTEST=MYVALUE;%(PreprocessorDefinitions)
          </PreprocessorDefinitions>
        </ResourceCompile>"""

    props_file = load(props_file)  # Remove all blanks and CR to compare
    props_file = "".join(s.strip() for s in props_file.splitlines())
    assert "".join(s.strip() for s in expected.splitlines()) in props_file


@pytest.mark.parametrize("mode,expected_toolset", [
    ("icx", "Intel C++ Compiler 2021"),
    ("dpcpp", "Intel(R) oneAPI DPC++ Compiler"),
    ("classic", "Intel C++ Compiler 19.2")
])
def test_msbuild_and_intel_cc_props(mode, expected_toolset):
    test_folder = temp_folder()
    settings = Settings({"build_type": ["Release"],
                         "compiler": {"intel-cc": {"version": ["2021.3"], "mode": [mode]},
                                      "msvc": {"version": ["193"], "cppstd": ["20"]}},
                         "os": ["Windows"],
                         "arch": ["x86_64"]})
    conanfile = ConanFile()
    conanfile.folders.set_base_generators(test_folder)
    conanfile.conf = Conf()
    conanfile.conf.define("tools.intel:installation_path", "my/intel/oneapi/path")
    conanfile.conf.define("tools.microsoft.msbuild:installation_path", ".")
    conanfile.settings = "os", "compiler", "build_type", "arch"
    conanfile.settings = settings
    conanfile.settings.build_type = "Release"
    conanfile.settings.compiler = "intel-cc"
    conanfile.settings.compiler.version = "2021.3"
    conanfile.settings.compiler.mode = mode
    conanfile.settings.os = "Windows"
    conanfile.settings.arch = "x86_64"

    msbuild = MSBuildToolchain(conanfile)
    props_file = os.path.join(test_folder, 'conantoolchain_release_x64.props')
    msbuild.generate()
    assert '<PlatformToolset>%s</PlatformToolset>' % expected_toolset in load(props_file)


@pytest.mark.parametrize("compiler,expected", [
    ("msvc", True),
    ("clang", False)
])
def test_is_msvc(compiler, expected):
    settings = Settings({"build_type": ["Release"],
                         "compiler": {compiler: {"version": ["2022"]}},
                         "os": ["Windows"],
                         "arch": ["x86_64"]})
    conanfile = ConanFile()
    conanfile.settings = settings
    conanfile.settings.compiler = compiler
    assert is_msvc(conanfile) == expected


def test_is_msvc_build():
    settings = Settings({"build_type": ["Release"],
                         "compiler": ["gcc", "msvc"],
                         "os": ["Windows"],
                         "arch": ["x86_64"]})
    conanfile = ConanFile()
    conanfile.settings = settings
    conanfile.settings.compiler = "gcc"
    conanfile.settings_build = conanfile.settings.copy()
    conanfile.settings_build.compiler = "msvc"
    assert is_msvc(conanfile) is False
    assert is_msvc(conanfile, build_context=True) is True


@pytest.mark.parametrize("compiler,shared,runtime,build_type,expected", [
    ("msvc", True, "static", "Release", True),
    ("msvc", True, "static", "Debug", True),
    ("clang", True, None, "Debug", False),
])
def test_is_msvc_static_runtime(compiler, shared, runtime, build_type, expected):
    options = MockOptions({"shared": shared})
    settings = MockSettings({"build_type": build_type,
                             "arch": "x86_64",
                             "compiler": compiler,
                             "compiler.runtime": runtime,
                             "compiler.version": "17",
                             "cppstd": "17"})
    conanfile = ConanFileMock(settings, options)
    assert is_msvc_static_runtime(conanfile) == expected


def test_msbuildtoolchain_changing_flags_via_attributes():
    test_folder = temp_folder()
    settings = Settings({"build_type": ["Release"],
                         "compiler": {"msvc": {"version": ["193"], "cppstd": ["20"]}},
                         "os": ["Windows"],
                         "arch": ["x86_64"]})
    conanfile = ConanFile()
    conanfile.settings = settings
    conanfile.folders.set_base_generators(test_folder)
    conanfile.conf = Conf()
    conanfile.conf.define_path("tools.microsoft.msbuild:installation_path", ".")
    conanfile.settings_build = settings
    conanfile.settings.build_type = "Release"
    conanfile.settings.compiler = "msvc"
    conanfile.settings.compiler.version = "193"
    conanfile.settings.compiler.cppstd = "20"
    conanfile.settings.os = "Windows"
    conanfile.settings.arch = "x86_64"

    msbuild = MSBuildToolchain(conanfile)
    msbuild.cxxflags.append("/flag1")
    msbuild.cflags.append("/flag2")
    msbuild.ldflags.append("/link1")
    msbuild.generate()
    toolchain = load(os.path.join(test_folder, "conantoolchain_release_x64.props"))

    expected_cl_compile = """
    <ClCompile>
      <PreprocessorDefinitions>%(PreprocessorDefinitions)</PreprocessorDefinitions>
      <AdditionalOptions>/flag1 /flag2 %(AdditionalOptions)</AdditionalOptions>"""
    expected_link = """
    <Link>
      <AdditionalOptions>/link1 %(AdditionalOptions)</AdditionalOptions>
    </Link>"""
    expected_resource_compile = """
    <ResourceCompile>
      <PreprocessorDefinitions>%(PreprocessorDefinitions)</PreprocessorDefinitions>
    </ResourceCompile>"""
    assert expected_cl_compile in toolchain
    assert expected_link in toolchain
    assert expected_resource_compile in toolchain



from conan.test.assets.genconanfile import GenConanfile
from conan.test.utils.tools import TestClient


def test_build_policies_in_conanfile():
    client = TestClient(default_server_user=True, light=True)
    base = GenConanfile("hello0", "1.0").with_exports("*")
    conanfile = str(base) + "\n    build_policy = 'missing'"
    client.save({"conanfile.py": conanfile})
    client.run("export . --user=lasote --channel=stable")

    # Install, it will build automatically if missing (without the --build missing option)
    client.run("install --requires=hello0/1.0@lasote/stable")
    assert "Building" in client.out

    # Try to do it again, now we have the package, so no build is done
    client.run("install --requires=hello0/1.0@lasote/stable")
    assert "Building" not in client.out

    # Try now to upload all packages, should not crash because of the "missing" build policy
    client.run("upload hello0/1.0@lasote/stable -r default")

    #  --- Build policy to always ---
    conanfile = str(base) + "\n    build_policy = 'always'"
    client.save({"conanfile.py": conanfile}, clean_first=True)
    client.run("export . --user=lasote --channel=stable")

    # Install, it will build automatically if missing (without the --build missing option)
    client.run("install --requires=hello0/1.0@lasote/stable", assert_error=True)
    assert "ERROR: hello0/1.0@lasote/stable: build_policy='always' has been removed" in client.out


def test_build_policy_missing():
    c = TestClient(default_server_user=True, light=True)
    conanfile = GenConanfile("pkg", "1.0").with_class_attribute('build_policy = "missing"')\
                                          .with_class_attribute('upload_policy = "skip"')
    c.save({"conanfile.py": conanfile})
    c.run("export .")

    # the --build=never has higher priority
    c.run("install --requires=pkg/1.0@ --build=never", assert_error=True)
    assert "ERROR: Missing prebuilt package for 'pkg/1.0'" in c.out

    c.run("install --requires=pkg/1.0@")
    assert "pkg/1.0: Building package from source as defined by build_policy='missing'" in c.out

    # If binary already there it should do nothing
    c.run("install --requires=pkg/1.0@")
    assert "pkg/1.0: Building package from source" not in c.out

    c.run("upload * -r=default -c")
    assert "Uploading package" not in c.out
    assert "pkg/1.0: Skipping upload of binaries, because upload_policy='skip'" in c.out



import pytest

from conan.internal.graph.build_mode import BuildMode
from conan.errors import ConanException
from conan.api.model import RecipeReference
from conan.test.utils.mocks import ConanFileMock, RedirectedTestOutput
from conan.test.utils.tools import redirect_output


@pytest.fixture
def conanfile():
    return ConanFileMock(None)


def test_skip_package(conanfile):
    build_mode = BuildMode(["!zlib/*", "other*"])
    assert not build_mode.forced(conanfile, RecipeReference.loads("zlib/1.2.11#23423423"))
    assert build_mode.forced(conanfile, RecipeReference.loads("other/1.2"))

    build_mode = BuildMode(["!zlib/*", "*"])
    assert not build_mode.forced(conanfile, RecipeReference.loads("zlib/1.2.11#23423423"))
    assert build_mode.forced(conanfile, RecipeReference.loads("other/1.2"))

    build_mode = BuildMode(["!zlib/*"])
    assert not build_mode.forced(conanfile, RecipeReference.loads("zlib/1.2.11#23423423"))
    assert not build_mode.forced(conanfile, RecipeReference.loads("other/1.2"))


def test_valid_params():
    build_mode = BuildMode(["missing"])
    assert build_mode.missing is True
    assert build_mode.never is False
    assert build_mode.cascade is False

    build_mode = BuildMode(["never"])
    assert build_mode.missing is False
    assert build_mode.never is True
    assert build_mode.cascade is False

    build_mode = BuildMode(["cascade"])
    assert build_mode.missing is False
    assert build_mode.never is False
    assert build_mode.cascade is True


def test_invalid_configuration():
    for mode in ["missing", "cascade"]:
        with pytest.raises(ConanException, match=r"--build=never not compatible "
                                                 r"with other options"):
            BuildMode([mode, "never"])


def test_common_build_force(conanfile):
    output = RedirectedTestOutput()
    with redirect_output(output):
        reference = RecipeReference.loads("hello/0.1@user/testing")
        build_mode = BuildMode(["hello/*"])
        assert build_mode.forced(conanfile, reference) is True


def test_no_user_channel(conanfile):
    output = RedirectedTestOutput()
    with redirect_output(output):
        reference = RecipeReference.loads("hello/0.1@")
        build_mode = BuildMode(["hello/0.1@"])
        assert build_mode.forced(conanfile, reference) is True


def test_revision_included(conanfile):
    output = RedirectedTestOutput()
    with redirect_output(output):
        reference = RecipeReference.loads("hello/0.1@user/channel#rrev1")
        build_mode = BuildMode(["hello/0.1@user/channel#rrev1"])
        assert build_mode.forced(conanfile, reference) is True


def test_no_user_channel_revision_included(conanfile):
    output = RedirectedTestOutput()
    with redirect_output(output):
        reference = RecipeReference.loads("hello/0.1@#rrev1")
        build_mode = BuildMode(["hello/0.1@#rrev1"])
        assert build_mode.forced(conanfile, reference) is True


def test_non_matching_build_force(conanfile):
    output = RedirectedTestOutput()
    with redirect_output(output):
        reference = RecipeReference.loads("Bar/0.1@user/testing")
        build_mode = BuildMode(["hello"])
        assert build_mode.forced(conanfile, reference) is False


def test_full_reference_build_force(conanfile):
    output = RedirectedTestOutput()
    with redirect_output(output):
        reference = RecipeReference.loads("bar/0.1@user/testing")
        build_mode = BuildMode(["bar/0.1@user/testing"])
        assert build_mode.forced(conanfile, reference) is True


def test_non_matching_full_reference_build_force(conanfile):
    output = RedirectedTestOutput()
    with redirect_output(output):
        reference = RecipeReference.loads("bar/0.1@user/stable")
        build_mode = BuildMode(["bar/0.1@user/testing"])
        assert build_mode.forced(conanfile, reference) is False


def test_multiple_builds(conanfile):
    output = RedirectedTestOutput()
    with redirect_output(output):
        reference = RecipeReference.loads("bar/0.1@user/stable")
        build_mode = BuildMode(["bar/*", "Foo/*"])
        assert build_mode.forced(conanfile, reference) is True


def test_allowed(conanfile):
    build_mode = BuildMode(["missing"])
    assert build_mode.allowed(conanfile) is True

    build_mode = BuildMode(None)
    assert build_mode.allowed(conanfile) is False


def test_casing(conanfile):
    output = RedirectedTestOutput()
    with redirect_output(output):
        reference = RecipeReference.loads("boost/1.69.0@user/stable")

        build_mode = BuildMode(["boost*"])
        assert build_mode.forced(conanfile, reference) is True
        build_mode = BuildMode(["bo*"])
        assert build_mode.forced(conanfile, reference) is True

        output.clear()
        build_mode = BuildMode(["Boost*"])
        assert build_mode.forced(conanfile, reference) is False
        build_mode = BuildMode(["Bo*"])
        assert build_mode.forced(conanfile, reference) is False


def test_pattern_matching(conanfile):
    output = RedirectedTestOutput()
    with redirect_output(output):
        build_mode = BuildMode(["boost*"])
        reference = RecipeReference.loads("boost/1.69.0@user/stable")
        assert (build_mode.forced(conanfile, reference)) is True
        reference = RecipeReference.loads("boost_addons/1.0.0@user/stable")
        assert (build_mode.forced(conanfile, reference)) is True
        reference = RecipeReference.loads("myboost/1.0@user/stable")
        assert (build_mode.forced(conanfile, reference)) is False
        reference = RecipeReference.loads("foo/boost@user/stable")
        assert (build_mode.forced(conanfile, reference)) is False
        reference = RecipeReference.loads("foo/1.0@boost/stable")
        assert (build_mode.forced(conanfile, reference)) is False
        reference = RecipeReference.loads("foo/1.0@user/boost")
        assert build_mode.forced(conanfile, reference) is False

        build_mode = BuildMode(["foo/*@user/stable"])
        reference = RecipeReference.loads("foo/1.0.0@user/stable")
        assert build_mode.forced(conanfile, reference) is True
        reference = RecipeReference.loads("foo/1.0@user/stable")
        assert build_mode.forced(conanfile, reference) is True
        reference = RecipeReference.loads("foo/1.0.0-abcdefg@user/stable")
        assert build_mode.forced(conanfile, reference) is True

        build_mode = BuildMode(["*@user/stable"])
        reference = RecipeReference.loads("foo/1.0.0@user/stable")
        assert build_mode.forced(conanfile, reference) is True
        reference = RecipeReference.loads("bar/1.0@user/stable")
        assert build_mode.forced(conanfile, reference) is True
        reference = RecipeReference.loads("foo/1.0.0-abcdefg@user/stable")
        assert build_mode.forced(conanfile, reference) is True
        reference = RecipeReference.loads("foo/1.0.0@NewUser/stable")
        assert build_mode.forced(conanfile, reference) is False

        build_mode = BuildMode(["*tool*"])
        reference = RecipeReference.loads("tool/0.1@lasote/stable")
        assert build_mode.forced(conanfile, reference) is True
        reference = RecipeReference.loads("pythontool/0.1@lasote/stable")
        assert build_mode.forced(conanfile, reference) is True
        reference = RecipeReference.loads("sometool/1.2@user/channel")
        assert build_mode.forced(conanfile, reference) is True

        build_mode = BuildMode(["tool/*"])
        reference = RecipeReference.loads("tool/0.1@lasote/stable")
        assert build_mode.forced(conanfile, reference) is True
        reference = RecipeReference.loads("tool/1.1@user/testing")
        assert build_mode.forced(conanfile, reference) is True
        reference = RecipeReference.loads("pythontool/0.1@lasote/stable")
        assert build_mode.forced(conanfile, reference) is False



import json
import textwrap

from conan.test.utils.tools import TestClient


def test_cpp_info_serialize_round_trip():
    """ test that serialize and deserialize CppInfo works
    """
    # TODO: Define standard name for file
    c = TestClient()
    conanfile = textwrap.dedent("""\
        import os
        from conan import ConanFile
        from conan.tools import CppInfo

        class Pkg(ConanFile):
            name = "pkg"
            version = "0.1"

            def package(self):
                cpp_info = CppInfo(self)
                cpp_info.includedirs = ["myinc"]
                cpp_info.libs = ["mylib", "myother"]
                cpp_info.libdirs = ["mylibs"]
                p = os.path.join(self.package_folder, "cpp_info.json")
                cpp_info.save(p)

            def package_info(self):
                cpp_info = CppInfo(self).load("cpp_info.json")
                self.cpp_info = cpp_info
        """)

    c.save({"conanfile.py": conanfile})
    c.run("create . --format=json")
    graph = json.loads(c.stdout)
    cpp_info = graph["graph"]["nodes"]["1"]["cpp_info"]["root"]
    assert cpp_info["includedirs"][0].endswith("myinc")
    assert cpp_info["libdirs"][0].endswith("mylibs")
    assert cpp_info["libs"] == ["mylib", "myother"]



import os

from conan.api.output import ConanOutput

from conan.internal.cache.home_paths import HomePaths
from conan.internal.conan_app import ConanApp
from conan.internal.graph.graph import CONTEXT_HOST, RECIPE_VIRTUAL, Node
from conan.internal.graph.graph_builder import DepsGraphBuilder
from conan.internal.graph.profile_node_definer import consumer_definer
from conan.errors import ConanException

from conan.internal.model.conanconfig import loadconanconfig, saveconanconfig, loadconanconfig_yml
from conan.internal.model.conf import BUILT_IN_CONFS
from conan.internal.model.pkg_type import PackageType
from conan.api.model import RecipeReference, Remote
from conan.internal.util.files import rmdir, remove


class ConfigAPI:
    """ This API provides methods to manage the Conan configuration in the Conan home folder.
    It allows installing configurations from various sources, retrieving global configuration
    values, and listing available configurations. It also provides methods to clean the
    Conan home folder, resetting it to a clean state.
    """

    def __init__(self, conan_api, helpers):
        self._conan_api = conan_api
        self._helpers = helpers

    def home(self):
        """ return the current Conan home folder containing the configuration files like
        remotes, settings, profiles, and the packages cache. It is provided for debugging
        purposes. Recall that it is not allowed to write, modify or remove packages in the
        packages cache, and that to automate tasks that uses packages from the cache Conan
        provides mechanisms like deployers or custom commands.
        """
        return self._conan_api.cache_folder

    def install(self, path_or_url, verify_ssl, config_type=None, args=None,
                source_folder=None, target_folder=None):
        """ install Conan configuration from a git repo, from a zip file in an http server
        or a local folder
        """
        from conan.internal.api.config.config_installer import configuration_install
        cache_folder = self._conan_api.cache_folder
        requester = self._helpers.requester
        configuration_install(cache_folder, requester, path_or_url, verify_ssl,
                              config_type=config_type, args=args,
                              source_folder=source_folder, target_folder=target_folder)
        self._conan_api.reinit()

    def install_package(self, require, lockfile=None, force=False, remotes=None, profile=None):
        ConanOutput().warning("The 'conan config install-pkg' is experimental",
                              warn_tag="experimental")
        require = RecipeReference.loads(require)
        required_pkgs = self.fetch_packages([require], lockfile, remotes, profile)
        installed_refs = self._install_pkgs(required_pkgs, force)
        self._conan_api.reinit()
        return installed_refs

    @staticmethod
    def load_conanconfig(path, remotes):
        if os.path.isdir(path):
            path = os.path.join(path, "conanconfig.yml")
        requested_requires, urls = loadconanconfig_yml(path)
        if urls:
            new_remotes = [Remote(f"config_install_url{'_' + str(i)}", url=url)
                           for i, url in enumerate(urls)]
            remotes = remotes or []
            remotes += new_remotes
        return requested_requires, remotes

    def install_conanconfig(self, path, lockfile=None, force=False, remotes=None, profile=None):
        ConanOutput().warning("The 'conan config install-pkg' is experimental",
                              warn_tag="experimental")
        requested_requires, remotes = self.load_conanconfig(path, remotes)
        required_pkgs = self.fetch_packages(requested_requires, lockfile, remotes, profile)
        installed_refs = self._install_pkgs(required_pkgs, force)
        self._conan_api.reinit()
        return installed_refs

    def _install_pkgs(self, required_pkgs, force):
        out = ConanOutput()
        out.title("Configuration packages to install")
        config_version_file = HomePaths(self._conan_api.home_folder).config_version_path
        if not os.path.exists(config_version_file):
            config_versions = []
        else:
            ConanOutput().info(f"Reading existing config-versions file: {config_version_file}")
            config_versions = loadconanconfig(config_version_file)
        config_versions_dict = {r.name: r for r in config_versions}
        if len(config_versions_dict) < len(config_versions):
            raise ConanException("There are multiple requirements for the same package "
                                 f"with different versions: {config_version_file}")

        new_config = config_versions_dict.copy()
        for required_pkg in required_pkgs:
            new_config.pop(required_pkg.ref.name, None)  # To ensure new order
            new_config[required_pkg.ref.name] = required_pkg.ref
        final_config_refs = [r for r in new_config.values()]

        prev_refs = "\n\t".join(repr(r) for r in config_versions)
        out.info(f"Previously installed configuration packages:\n\t{prev_refs}")

        new_refs = "\n\t".join(r.repr_notime() for r in final_config_refs)
        out.info(f"New configuration packages to install:\n\t{new_refs}")

        if list(config_versions_dict) == list(new_config)[:len(config_versions_dict)]:
            # There is no conflict in order, can be done safely
            if final_config_refs == config_versions:
                if force:
                    out.warning("The requested configurations are identical to the already "
                                "installed ones, but forcing re-installation because --force")
                    to_install = required_pkgs
                else:
                    out.info("The requested configurations are identical to the already "
                             "installed ones, skipping re-installation")
                    to_install = []
            else:
                out.info("Installing new or updating configuration packages")
                to_install = required_pkgs
        else:
            # Change in order of existing configuration
            if force:
                out.warning("Installing these configuration packages will break the "
                            "existing order, with possible side effects. "
                            "Forcing the installation because --force was defined", warn_tag="risk")
                to_install = required_pkgs
            else:
                msg = ("Installing these configuration packages will break the "
                       "existing order, with possible side effects, like breaking 'package_ids'.\n"
                       "If you still want to enforce this configuration you can:\n"
                       "   Use 'conan config clean' first to fully reset your configuration.\n"
                       "   Or use 'conan config install-pkg --force' to force installation.")
                raise ConanException(msg)

        out.title("Installing configuration from packages")
        # install things and update the Conan cache "config_versions.json" file
        from conan.internal.api.config.config_installer import configuration_install
        cache_folder = self._conan_api.cache_folder
        requester = self._helpers.requester
        for pkg in to_install:
            out.info(f"Installing configuration from {pkg.ref}")
            configuration_install(cache_folder, requester, uri=pkg.conanfile.package_folder,
                                  verify_ssl=False, config_type="dir",
                                  ignore=["conaninfo.txt", "conanmanifest.txt"])

        saveconanconfig(config_version_file, final_config_refs)
        return final_config_refs

    def fetch_packages(self, refs, lockfile=None, remotes=None, profile=None):
        """ install configuration stored inside a Conan package
        The installation of configuration will reinitialize the full ConanAPI
        """
        conan_api = self._conan_api
        remotes = conan_api.remotes.list() if remotes is None else remotes
        profile_host = profile_build = profile or conan_api.profiles.get_profile([])

        app = ConanApp(self._conan_api)

        ConanOutput().title("Fetching requested configuration packages")
        result = []
        for ref in refs:
            # Computation of a very simple graph that requires "ref"
            # Need to convert input requires to RecipeReference
            conanfile = app.loader.load_virtual(requires=[ref])
            consumer_definer(conanfile, profile_host, profile_build)
            root_node = Node(ref=None, conanfile=conanfile, context=CONTEXT_HOST,
                             recipe=RECIPE_VIRTUAL)
            root_node.is_conf = True
            update = ["*"]
            builder = DepsGraphBuilder(app.proxy, app.loader, app.range_resolver, app.cache, remotes,
                                       update, update, self._helpers.global_conf)
            deps_graph = builder.load_graph(root_node, profile_host, profile_build, lockfile)

            # Basic checks of the package: correct package_type and no-dependencies
            deps_graph.report_graph_error()
            pkg = deps_graph.root.edges[0].dst
            ConanOutput().info(f"Configuration from package: {pkg}")
            if pkg.conanfile.package_type is not PackageType.CONF:
                raise ConanException(f'{pkg.conanfile} is not of package_type="configuration"')
            if pkg.edges:
                raise ConanException(f"Configuration package {pkg.ref} cannot have dependencies")

            # The computation of the "package_id" and the download of the package is done as usual
            # By default we allow all remotes, and build_mode=None, always updating
            conan_api.graph.analyze_binaries(deps_graph, None, remotes, update=update,
                                             lockfile=lockfile)
            conan_api.install.install_binaries(deps_graph=deps_graph, remotes=remotes)
            result.append(pkg)
        return result

    def get(self, name, default=None, check_type=None):
        """ get the value of a global.conf item
        """
        return self._helpers.global_conf.get(name, default=default, check_type=check_type)

    def show(self, pattern) -> dict:
        """ get the values of global.conf for those configurations that matches the pattern
        """
        return self._helpers.global_conf.show(pattern)

    @staticmethod
    def conf_list():
        """ list all the available built-in configurations
        """
        return BUILT_IN_CONFS.copy()

    def clean(self):
        """ reset the Conan home folder to a clean state, removing all the user
        custom configuration, custom files, and resetting modified files
        """
        contents = os.listdir(self.home())
        packages_folder = (self._helpers.global_conf.get("core.cache:storage_path") or
                           os.path.join(self.home(), "p"))
        for content in contents:
            content_path = os.path.join(self.home(), content)
            if content_path == packages_folder:
                continue
            ConanOutput().debug(f"Removing {content_path}")
            if os.path.isdir(content_path):
                rmdir(content_path)
            else:
                remove(content_path)
        self._conan_api.reinit()
        # CHECK: This also generates a remotes.json that is not there after a conan profile show?
        self._conan_api.migrate()

    @property
    def settings_yml(self):
        """ Get the contents of the settings.yml and user_settings.yml files,
            which define the possible values for settings.

            Note that this is different from the settings present in a conanfile,
            which represent the actual values for a specific package, while this
            property represents the possible values for each setting.

            :returns: A read-only object representing the settings scheme, with a
                ``possible_values()`` method that returns a dictionary with the possible values for each setting,
                and a ``fields`` property that returns an ordered list with the fields of each setting.
                Note that it's possible to access nested settings using attribute access,
                such as ``settings_yml.compiler.possible_values()``.
        """

        class SettingsYmlInterface:
            def __init__(self, settings):
                self._settings = settings

            def possible_values(self):
                """ returns a dict with the possible values for each setting """
                return self._settings.possible_values()

            @property
            def fields(self):
                """ returns a dict with the fields of each setting """
                return self._settings.fields

            def __getattr__(self, item):
                return SettingsYmlInterface(getattr(self._settings, item))

            def __str__(self):
                return str(self._settings)

        return SettingsYmlInterface(self._helpers.settings_yml)



from conan.internal.paths import CONANFILE
from conan.test.assets.genconanfile import GenConanfile
from conan.test.utils.tools import TestClient, TestServer, TestRequester


class TestDownloadRetries:

    def test_recipe_download_retry(self):
        test_server = TestServer()
        client = TestClient(servers={"default": test_server}, inputs=["admin", "password"])

        client.save({CONANFILE: GenConanfile()})
        client.run("create . --name=pkg --version=0.1 --user=lasote --channel=stable")
        client.run("upload '*' -c -r default")

        class Response:
            def __init__(self, ok, status_code):
                self.ok = ok
                self.status_code = status_code

        class BuggyRequester(TestRequester):
            def get(self, *args, **kwargs):
                if "files/conanfile.py" not in args[0]:
                    return super(BuggyRequester, self).get(*args, **kwargs)
                else:
                    return Response(False, 200)

        # The buggy requester will cause a failure only downloading files, not in regular requests
        client = TestClient(servers={"default": test_server}, inputs=["admin", "password"],
                            requester_class=BuggyRequester)
        client.run("install --requires=pkg/0.1@lasote/stable", assert_error=True)
        assert str(client.out).count("Waiting 0 seconds to retry...") == 2
        assert str(client.out).count("Error 200 downloading") == 3



import os
import textwrap

from conan.internal.model.manifest import FileTreeManifest
from conan.test.assets.genconanfile import GenConanfile
from conan.test.utils.tools import TestClient
from conan.internal.util.files import save


def test_post_package():
    """ Test that 'post_package' hook is called before computing the manifest
    """
    t = TestClient()
    complete_hook = textwrap.dedent("""\
        import os
        from conan.tools.files import save
        def post_package(conanfile):
            save(conanfile, os.path.join(conanfile.package_folder, "myfile.txt"), "content!!")
        """)
    hook_path = os.path.join(t.paths.hooks_path, "complete_hook", "hook_complete.py")
    save(hook_path, complete_hook)
    t.save({'conanfile.py': GenConanfile("pkg", "0.1")})
    t.run("create .")
    pref_layout = t.created_layout()
    manifest = FileTreeManifest.load(pref_layout.package())
    assert "myfile.txt" in manifest.file_sums

    t.run("remove * -c")
    t.run("export-pkg .")
    pref_layout = t.created_layout()
    manifest = FileTreeManifest.load(pref_layout.package())
    assert "myfile.txt" in manifest.file_sums



import cProfile
import os
import pstats
import time
from pstats import SortKey
import pytest

from conan.test.assets.genconanfile import GenConanfile
from conan.test.utils.tools import TestClient


@pytest.mark.skip(reason="This is a performance test, skip for normal runs")
def test_large_graph():
    c = TestClient(cache_folder=os.path.join(os.path.dirname(__file__), "cache"))
    num_test = 40
    num_pkgs = 40

    """for i in range(num_test):
        conanfile = GenConanfile(f"test{i}", "0.1")
        if i > 0:
            conanfile.with_requires(f"test{i-1}/0.1")
        c.save({"conanfile.py": conanfile})
        c.run("create .")

    for i in range(num_pkgs):
        conanfile = GenConanfile(f"pkg{i}", "0.1").with_test_requires(f"test{num_test-1}/0.1")
        if i > 0:
            conanfile.with_requires(f"pkg{i-1}/0.1")
        c.save({"conanfile.py": conanfile})
        c.run("create .")

    """
    t = time.time()
    pr = cProfile.Profile()
    pr.enable()
    c.run(f"install --requires=pkg{num_pkgs - 1}/0.1")
    pr.disable()
    print(time.time()-t)

    sortby = SortKey.CUMULATIVE
    ps = pstats.Stats(pr).sort_stats(sortby)
    ps.print_stats()

    #graph = json.loads(c.stdout)
    #assert len(graph["graph"]["nodes"]) == 1 + num_pkgs + num_test * num_pkgs


@pytest.mark.skip(reason="This is a performance test, skip for normal runs")
def test_large_graph2():
    c = TestClient(cache_folder=os.path.join(os.path.dirname(__file__), "cache"))
    num_test = 20
    num_pkgs = 20
    branches = ["a", "b", "c"]

    c.save({"conanfile.py": GenConanfile("testbase", "0.1")})
    c.run("export .")
    for i in range(num_test):
        for branch in branches:
            conanfile = GenConanfile(f"test{branch}{i}", "0.1")
            if i > 0:
                conanfile.with_requires(f"test{branch}{i-1}/0.1", "testbase/0.1")
            else:
                conanfile.with_requires("testbase/0.1")
            c.save({"conanfile.py": conanfile})
            c.run("export .")

    for i in range(num_pkgs):
        conanfile = GenConanfile(f"pkg{i}", "0.1")
        for branch in branches:
            conanfile.with_test_requires(f"test{branch}{num_test-1}/0.1")
        if i > 0:
            conanfile.with_requires(f"pkg{i-1}/0.1")
        c.save({"conanfile.py": conanfile})
        c.run("export .")

    t = time.time()
    pr = cProfile.Profile()
    pr.enable()
    c.run(f"graph info --requires=pkg{num_pkgs - 1}/0.1")
    pr.disable()
    print(time.time()-t)

    sortby = SortKey.CUMULATIVE
    ps = pstats.Stats(pr).sort_stats(sortby)
    ps.print_stats()



import os
import platform

import pytest

from conan.test.assets.genconanfile import GenConanfile
from conan.test.utils.tools import TestClient

"""
When we use the VirtualRunEnv and VirtualBuildEnd generators, we take information from the
self.dependencies env_buildinfo and env_runinfo, but to format correctly the environment
variables that are paths to run in a windows bash, we need to look at the consumer conanfile
not the dependency conanfile. This is testing that in the process of aggregating the paths,
these are correct.
"""


@pytest.fixture
def client():
    client = TestClient()
    conanfile = str(GenConanfile())
    conanfile += """

    def package_info(self):
        self.buildenv_info.define_path("AR", "c:/path/to/ar")
        self.buildenv_info.append_path("PATH", "c:/path/to/something")
        self.runenv_info.define_path("RUNTIME_VAR", "c:/path/to/exe")
    """
    client.save({"conanfile.py": conanfile})
    client.run("create . --name=foo --version=1.0")
    client.save_home({"global.conf": "tools.microsoft.bash:subsystem=cygwin"})
    return client


@pytest.mark.parametrize("win_bash", [True, False, None])
@pytest.mark.skipif(platform.system() != "Windows", reason="Requires Windows")
def test_virtualenv_deactivated(client, win_bash):
    conanfile = GenConanfile().with_settings("os").with_require("foo/1.0")
    if win_bash:
        conanfile.with_class_attribute("win_bash = True")

    client.save({"conanfile.py": conanfile})
    client.run("install . -s:b os=Windows -s:h os=Windows")

    if win_bash:
        # Assert there is no "bat" files generated because the environment can and will be run inside
        # the bash
        assert not os.path.exists(os.path.join(client.current_folder, "conanbuildenv.bat"))
        build_contents = client.load("conanbuildenv.sh")
        assert "/cygdrive/c/path/to/ar" in build_contents
        assert "$PATH:/cygdrive/c/path/to/something" in build_contents
    else:
        assert not os.path.exists(os.path.join(client.current_folder, "conanbuildenv.sh"))
        build_contents = client.load("conanbuildenv.bat")
        assert "c:/path/to/ar" in build_contents
        assert "c:/path/to/something" in build_contents

    assert not os.path.exists(os.path.join(client.current_folder, "conanrunenv.sh"))
    run_contents = client.load("conanrunenv.bat")
    assert "c:/path/to/exe" in run_contents


@pytest.mark.skipif(platform.system() != "Windows", reason="Requires Windows")
def test_nowinbash(client):
    """ whe the recipe doesn't define win_bash=True 2 things can happen:
    - No need to run in bash, things will run in normal command, with .bat
    - You run directly in a subsytem terminal a recipe that was not aware of that, still the
       files need to follow the subsystem
    """
    conanfile = str(GenConanfile().with_settings("os")
                    .with_generator("VirtualBuildEnv").with_generator("VirtualRunEnv")
                    .with_require("foo/1.0"))

    client.save({"conanfile.py": conanfile})
    client.run("install . -s:b os=Windows -s:h os=Windows")
    assert not os.path.exists(os.path.join(client.current_folder, "conanbuildenv.sh"))
    build_contents = client.load("conanbuildenv.bat")
    assert 'set "AR=c:/path/to/ar"' in build_contents
    assert 'set "PATH=%PATH%;c:/path/to/something"' in build_contents
    assert not os.path.exists(os.path.join(client.current_folder, "conanrunenv.sh"))
    run_contents = client.load("conanrunenv.bat")
    assert 'set "RUNTIME_VAR=c:/path/to/exe"' in run_contents

    # Running it inside a cygwin subsystem
    client.save({"conanfile.py": conanfile}, clean_first=True)
    client.run("install . -s:b os=Windows -s:h os=Windows "
               "-c:h tools.microsoft.bash:active=True")
    assert not os.path.exists(os.path.join(client.current_folder, "conanbuildenv.bat"))
    build_contents = client.load("conanbuildenv.sh")
    assert 'export AR="/cygdrive/c/path/to/ar"' in build_contents
    assert 'export PATH="$PATH:/cygdrive/c/path/to/something"' in build_contents
    assert not os.path.exists(os.path.join(client.current_folder, "conanrunenv.bat"))
    run_contents = client.load("conanrunenv.sh")
    assert 'export RUNTIME_VAR="/cygdrive/c/path/to/exe"' in run_contents

    # Running it inside a msys2 subsystem, test tat it overrides
    client.save({"conanfile.py": conanfile}, clean_first=True)
    client.run("install . -s:b os=Windows -s:h os=Windows "
               "-c tools.microsoft.bash:subsystem=msys2 -c:h tools.microsoft.bash:active=True")
    assert not os.path.exists(os.path.join(client.current_folder, "conanbuildenv.bat"))
    build_contents = client.load("conanbuildenv.sh")
    assert 'export AR="/c/path/to/ar"' in build_contents
    assert 'export PATH="$PATH:/c/path/to/something"' in build_contents
    assert not os.path.exists(os.path.join(client.current_folder, "conanrunenv.bat"))
    run_contents = client.load("conanrunenv.sh")
    assert 'export RUNTIME_VAR="/c/path/to/exe"' in run_contents



import os
import textwrap

from parameterized import parameterized

from conan.test.assets.genconanfile import GenConanfile
from conan.test.utils.tools import TestClient


class TestConfigureOptions:
    """
    Test config_options(), configure() and package_id() methods can manage shared, fPIC and
    header_only options automatically.
    """

    @parameterized.expand([
        ["Linux", False, False, False, [False, False, False]],
        ["Windows", False, False, False, [False, None, False]],
        ["Windows", True, False, False, [True, None, False]],
        ["Windows", False, False, True, [None, None, True]],
        ["Linux", False, False, True, [None, None, True]],
        ["Linux", True, True, False, [True, None, False]],
        ["Linux", True, False, False, [True, None, False]],
        ["Linux", True, True, True, [None, None, True]],
        ["Linux", True, True, True, [None, None, True]],
        ["Linux", False, True, False, [False, True, False]],
        ["Linux", False, True, False, [False, True, False]],
    ])
    def test_methods_not_defined(self, settings_os, shared, fpic, header_only, result):
        """
        Test that options are managed automatically when methods config_options and configure are not
        defined and implements = ["auto_shared_fpic", "auto_header_only"].
        Check that header only package gets its unique package ID.
        """
        client = TestClient()
        conanfile = textwrap.dedent(f"""\
           from conan import ConanFile

           class Pkg(ConanFile):
               settings = "os", "compiler", "arch", "build_type"
               options = {{"shared": [True, False], "fPIC": [True, False], "header_only": [True, False]}}
               default_options = {{"shared": {shared}, "fPIC": {fpic}, "header_only": {header_only}}}
               implements = ["auto_shared_fpic", "auto_header_only"]

               def build(self):
                   shared = self.options.get_safe("shared")
                   fpic = self.options.get_safe("fPIC")
                   header_only = self.options.get_safe("header_only")
                   self.output.info(f"shared: {{shared}}, fPIC: {{fpic}}, header only: {{header_only}}")
            """)
        client.save({"conanfile.py": conanfile})
        client.run(f"create . --name=pkg --version=0.1 -s os={settings_os}")
        result = f"shared: {result[0]}, fPIC: {result[1]}, header only: {result[2]}"
        assert result in client.out
        if header_only:
            assert "Package 'da39a3ee5e6b4b0d3255bfef95601890afd80709' created"in client.out

    @parameterized.expand([
        ["Linux", False, False, False, [False, False, False]],
        ["Linux", False, False, True, [False, False, True]],
        ["Linux", False, True, False, [False, True, False]],
        ["Linux", False, True, True, [False, True, True]],
        ["Linux", True, False, False, [True, False, False]],
        ["Linux", True, False, True, [True, False, True]],
        ["Linux", True, True, False, [True, True, False]],
        ["Linux", True, True, True, [True, True, True]],
        ["Windows", False, False, False, [False, False, False]],
        ["Windows", False, False, True, [False, False, True]],
        ["Windows", False, True, False, [False, True, False]],
        ["Windows", False, True, True, [False, True, True]],
        ["Windows", True, False, False, [True, False, False]],
        ["Windows", True, False, True, [True, False, True]],
        ["Windows", True, True, False, [True, True, False]],
        ["Windows", True, True, True, [True, True, True]],
    ])
    def test_optout(self, settings_os, shared, fpic, header_only, result):
        """
        Test that options are not managed automatically when methods are defined even if implements = ["auto_shared_fpic", "auto_header_only"]
        Check that header only package gets its unique package ID.
        """
        client = TestClient()
        conanfile = textwrap.dedent(f"""\
           from conan import ConanFile

           class Pkg(ConanFile):
               settings = "os", "compiler", "arch", "build_type"
               options = {{"shared": [True, False], "fPIC": [True, False], "header_only": [True, False]}}
               default_options = {{"shared": {shared}, "fPIC": {fpic}, "header_only": {header_only}}}
               implements = ["auto_shared_fpic", "auto_header_only"]

               def config_options(self):
                   pass

               def configure(self):
                   pass

               def build(self):
                   shared = self.options.get_safe("shared")
                   fpic = self.options.get_safe("fPIC")
                   header_only = self.options.get_safe("header_only")
                   self.output.info(f"shared: {{shared}}, fPIC: {{fpic}}, header only: {{header_only}}")
            """)
        client.save({"conanfile.py": conanfile})
        client.run(f"create . --name=pkg --version=0.1 -s os={settings_os}")
        result = f"shared: {result[0]}, fPIC: {result[1]}, header only: {result[2]}"
        assert result in client.out
        if header_only:
            assert "Package 'da39a3ee5e6b4b0d3255bfef95601890afd80709' created"in client.out

    def test_header_package_type_pid(self):
        """
        Test that we get the pid for header only when package type is set to header-library
        """
        client = TestClient()
        conanfile = textwrap.dedent(f"""\
               from conan import ConanFile

               class Pkg(ConanFile):
                   settings = "os", "compiler", "arch", "build_type"
                   package_type = "header-library"
                   implements = ["auto_shared_fpic", "auto_header_only"]

                """)
        client.save({"conanfile.py": conanfile})
        client.run(f"create . --name=pkg --version=0.1")
        assert "Package 'da39a3ee5e6b4b0d3255bfef95601890afd80709' created" in client.out


def test_config_options_override_behaviour():
    tc = TestClient(light=True)
    tc.save({"conanfile.py": textwrap.dedent("""
    from conan import ConanFile

    class Pkg(ConanFile):
        name = "pkg"
        version = "1.0"

        options = {"foo": [1, 2, 3]}
        default_options = {"foo": 1}

        def config_options(self):
            self.options.foo = 2

        def build(self):
            self.output.info(f"Foo: {self.options.foo}")
    """)})

    tc.run("create .")
    assert "Foo: 2" in tc.out

    tc.run("create . -o=&:foo=3")
    assert "Foo: 3" in tc.out


def test_configure_transitive_option():
    c = TestClient(light=True)
    c.save({"conanfile.py": GenConanfile("zlib", "0.1").with_shared_option(False)})
    c.run("create . -o *:shared=True")

    boost = textwrap.dedent("""
        from conan import ConanFile

        class BoostConan(ConanFile):
            name = "boostdbg"
            version = "1.0"
            options = {"shared": [True, False]}
            default_options ={"shared": False}
            requires = "zlib/0.1"

            def configure(self):
                self.options["zlib/*"].shared = self.options.shared
        """)

    c.save({"conanfile.py": boost}, clean_first=True)
    c.run("create .  -o boostdbg/*:shared=True")
    # It doesn't fail due to missing binary, as it assigned dependency shared=True
    pkg_folder = c.created_layout().package()
    conaninfo = c.load(os.path.join(pkg_folder, "conaninfo.txt"))
    assert "shared=True" in conaninfo



import json
import os
import textwrap

from conan.test.assets.genconanfile import GenConanfile
from conan.test.utils.tools import TestClient


def test_build_different_folders():
    conanfile = textwrap.dedent("""
        import os
        from conan import ConanFile

        class AConan(ConanFile):

            def build(self):
                self.output.warning("Build folder=>%s" % self.build_folder)
                self.output.warning("Src folder=>%s" % self.source_folder)
                assert(os.path.exists(self.build_folder))
                assert(os.path.exists(self.source_folder))
    """)

    client = TestClient()
    client.save({"conanfile.py": conanfile})
    with client.chdir("build1"):
        client.run("install ..")
    # Try relative to cwd
    client.run("build . --output-folder build2")
    assert "Build folder=>%s" % os.path.join(client.current_folder, "build2") in client.out
    assert "Src folder=>%s" % client.current_folder in client.out


def test_build_dots_names():
    client = TestClient()
    conanfile_dep = textwrap.dedent("""
        from conan import ConanFile

        class AConan(ConanFile):
            pass
    """)
    client.save({"conanfile.py": conanfile_dep})
    client.run("create . --name=hello.pkg --version=0.1 --user=lasote --channel=testing --format=json",
               redirect_stdout="hello.pkg.json")
    hellopkg_result = json.loads(client.load("hello.pkg.json"))
    client.run("create . --name=hello-tools --version=0.1 --user=lasote --channel=testing --format=json",
               redirect_stdout="hello-tools.json")
    hellotools_result = json.loads(client.load("hello-tools.json"))
    conanfile_scope_env = textwrap.dedent("""
        from conan import ConanFile

        class AConan(ConanFile):
            requires = "hello.pkg/0.1@lasote/testing", "hello-tools/0.1@lasote/testing"

            def generate(self):
                self.output.info("HELLO ROOT PATH: %s" %
                    self.dependencies["hello.pkg"].package_folder)
                self.output.info("HELLO ROOT PATH: %s" %
                    self.dependencies["hello-tools"].package_folder)

            def build(self):
                pass

    """)
    client.save({"conanfile.py": conanfile_scope_env}, clean_first=True)
    client.run("build conanfile.py --build=missing")

    assert hellopkg_result["graph"]["nodes"]["1"]["package_folder"] in client.out
    assert hellotools_result["graph"]["nodes"]["1"]["package_folder"] in client.out


def test_build_with_deps_env_info():
    client = TestClient()
    conanfile = textwrap.dedent("""
        from conan import ConanFile

        class AConan(ConanFile):
            name = "lib"
            version = "1.0"

            def package_info(self):
                self.buildenv_info.define("MYVAR", "23")

    """)
    client.save({"conanfile.py": conanfile})
    client.run("export . --user=lasote --channel=stable")

    conanfile = textwrap.dedent("""
        from conan import ConanFile
        from conan.tools.env import VirtualBuildEnv
        import os

        class AConan(ConanFile):
            build_requires = "lib/1.0@lasote/stable"

            def build(self):
                build_env = VirtualBuildEnv(self).vars()
                assert build_env.get("MYVAR") == "23"
                with build_env.apply():
                    assert(os.environ["MYVAR"] == "23")
    """)
    client.save({"conanfile.py": conanfile}, clean_first=True)
    client.run("build . --build missing")


def test_build_single_full_reference():
    client = TestClient()
    client.save({"conanfile.py": GenConanfile("foo", "1.0")})
    client.run("create . --build='*'")
    assert "foo/1.0: Forced build from source" in client.out


def test_build_multiple_full_reference():
    client = TestClient()
    client.save({"conanfile.py": GenConanfile("foo", "1.0")})
    client.run("create .")
    client.save({"conanfile.py": GenConanfile("bar", "1.0").with_requires("foo/1.0")})
    client.run("create --build foo/1.0@ --build bar/1.0@ .")
    assert "foo/1.0: Forced build from source" in client.out
    assert "bar/1.0: Forced build from source" in client.out


def test_debug_build_release_deps():
    # https://github.com/conan-io/conan/issues/2899
    client = TestClient()
    conanfile = textwrap.dedent("""
        from conan import ConanFile
        class Conan(ConanFile):
            name = "{name}"
            {requires}
            settings = "build_type"
            def build(self):
                self.output.info("BUILD: %s BuildType=%s!"
                                 % (self.name, self.settings.build_type))
            def package_info(self):
                self.output.info("PACKAGE_INFO: %s BuildType=%s!"
                                 % (self.name, self.settings.build_type))
        """)
    client.save({"conanfile.py": conanfile.format(name="dep", requires="")})
    client.run("create . --name=dep --version=0.1 --user=user --channel=testing -s build_type=Release")
    client.save({"conanfile.py": conanfile.format(name="mypkg", requires="requires = 'dep/0.1@user/testing'")})
    client.run("build . -s mypkg/*:build_type=Debug -s build_type=Release")
    assert "dep/0.1@user/testing: PACKAGE_INFO: dep BuildType=Release!" in client.out
    assert "conanfile.py (mypkg/None): BUILD: mypkg BuildType=Debug!" in client.out




import textwrap

from conan.test.assets.genconanfile import GenConanfile
from conan.test.utils.tools import TestClient


def test_timestamp_error():
    """ this test is a reproduction for
    # https://github.com/conan-io/conan/issues/11606

    It was crashing because of multiple test_requires, some of them being BINARY_SKIP,
    and the prev_timestamp was not being assigned by GraphBinariesAnalizer when caching
    """

    c = TestClient(default_server_user=True)
    engine = textwrap.dedent("""
        from conan import ConanFile

        class Engine(ConanFile):
            name = "engine"
            version = "0.1"
            def build_requirements(self):
                self.test_requires("gtest/0.1")
        """)
    app = textwrap.dedent("""
        from conan import ConanFile
        class App(ConanFile):
            def requirements(self):
                self.requires("engine/0.1")
            def build_requirements(self):
                self.test_requires("gtest/0.1")
        """)
    c.save({"gtest/conanfile.py": GenConanfile("gtest", "0.1"),
            "engine/conanfile.py": engine,
            "app/conanfile.py": app})
    c.run("create gtest")
    c.run("create engine")
    c.run("upload * -r=default -c")
    c.run("remove * -c")
    c.run("install app")
    # This used to fail, now it is not crashing anymore
    assert "Finalizing install" in c.out



import json

import pytest

from conan.test.utils.tools import TestClient, GenConanfile


class TestVersionRangeOverride:

    @pytest.fixture(autouse=True)
    def _setup(self):
        self.t = TestClient(light=True)
        self.t.save({"libb/conanfile.py": GenConanfile(),
                     "libc/conanfile.py":
                         GenConanfile().with_require("libb/[<=2.0]@user/channel")})
        self.t.run("export libb --name=libb --version=1.0 --user=user --channel=channel")
        self.t.run("export libb --name=libb --version=2.0 --user=user --channel=channel")
        self.t.run("export libb --name=libb --version=3.0 --user=user --channel=channel")
        self.t.run("export libc --name=libc --version=1.0 --user=user --channel=channel")

    def test(self):
        # Use the version range
        self.t.save({"conanfile.py": GenConanfile().with_require("libc/1.0@user/channel")})
        self.t.run("graph info . --filter requires")
        assert "libb/2.0@user/channel" in self.t.out

    def test_override_with_fixed_version(self):
        # Override upstream version range with a fixed version
        self.t.save({"conanfile.py": GenConanfile().with_requirement("libb/3.0@user/channel",
                                                                     override=True)
                                                   .with_require("libc/1.0@user/channel")})
        self.t.run("graph info . --filter requires")
        self.t.assert_overrides({'libb/[<=2.0]@user/channel': ['libb/3.0@user/channel']})
        assert "libb/3.0@user/channel#" in self.t.out

    def test_override_using_version_range(self):
        # Override upstream version range with a different (narrower) version range
        self.t.save({"conanfile.py": GenConanfile().with_requirement("libb/[<2.x]@user/channel",
                                                                     override=True)
                                                   .with_require("libc/1.0@user/channel")})
        self.t.run("graph info . --filter requires")
        self.t.assert_overrides({'libb/[<=2.0]@user/channel': ['libb/[<2.x]@user/channel']})
        assert "libb/2.0@user/channel" in self.t.out

    def test_override_version_range_outside(self):
        # Override upstream version range with a different (non intersecting) version range
        self.t.save({"conanfile.py": GenConanfile().with_requirement("libb/[>2.x]@user/channel",
                                                                     override=True)
                                                   .with_require("libc/1.0@user/channel")})
        self.t.run("graph info . --filter requires")
        self.t.assert_overrides({'libb/[<=2.0]@user/channel': ['libb/[>2.x]@user/channel']})
        assert "libb/3.0@user/channel" in self.t.out


class TestVersionRangeOverrideFail:

    def test_override(self):
        """
        pkga -> ros_perception  -> ros_core
           \\-----> pkgb  -----------/
        """
        # https://github.com/conan-io/conan/issues/8071
        t = TestClient(light=True)
        t.save({"conanfile.py": GenConanfile()})
        t.run("create . --name=ros_core --version=1.1.4 --user=3rdparty --channel=unstable")
        t.run("create . --name=ros_core --version=pr-53 --user=3rdparty --channel=snapshot")
        t.save({"conanfile.py": GenConanfile().with_requires("ros_core/1.1.4@3rdparty/unstable")})
        t.run("create . --name=ros_perception --version=1.1.4 --user=3rdparty --channel=unstable")
        t.run("create . --name=ros_perception --version=pr-53 --user=3rdparty --channel=snapshot")
        t.save({"conanfile.py": GenConanfile().with_requires("ros_core/[~1.1]@3rdparty/unstable")})
        t.run("create . --name=pkgb --version=0.1 --user=common --channel=unstable")
        t.save({"conanfile.py": GenConanfile("pkga", "0.1").with_requires(
            "ros_perception/[~1.1]@3rdparty/unstable",
            "pkgb/[~0]@common/unstable")})
        t.run("create . ")
        assert "ros_core/1.1.4@3rdparty/unstable" in t.out
        assert "ros_perception/1.1.4@3rdparty/unstable" in t.out
        assert "snapshot" not in t.out

        t.save({"conanfile.py": GenConanfile("pkga", "0.1")
               .with_require("pkgb/[~0]@common/unstable")
               .with_require("ros_perception/pr-53@3rdparty/snapshot")
               .with_requirement("ros_core/pr-53@3rdparty/snapshot", override=True)})

        t.run("create .  --build=missing --build=pkga")
        assert "ros_core/pr-53@3rdparty/snapshot" in t.out
        assert "ros_perception/pr-53@3rdparty/snapshot" in t.out

        # Override only the upstream without overriding the direct one
        t.save({"conanfile.py": GenConanfile("pkga", "0.1")
               .with_require("pkgb/[~0]@common/unstable")
               .with_require("ros_perception/[~1.1]@3rdparty/unstable")
               .with_requirement("ros_core/pr-53@3rdparty/snapshot", force=True)})

        t.run("create .  --build=missing --build=pkga")
        assert "ros_core/pr-53@3rdparty/snapshot" in t.out
        assert "ros_perception/1.1.4@3rdparty/unstable" in t.out

        # Check information got by graph info
        t.run("graph info . --format json")
        info = json.loads(t.stdout)
        expected_overrides = {
            "ros_core/[~1.1]@3rdparty/unstable": [
                "ros_core/pr-53@3rdparty/snapshot"
            ],
            "ros_core/1.1.4@3rdparty/unstable": [
                "ros_core/pr-53@3rdparty/snapshot"
            ]
        }
        assert info['graph']["overrides"] == expected_overrides
        expected_resolved_ranges = {
            "pkgb/[~0]@common/unstable": "pkgb/0.1@common/unstable",
            "ros_perception/[~1.1]@3rdparty/unstable": "ros_perception/1.1.4@3rdparty/unstable"
        }
        assert info['graph']["resolved_ranges"] == expected_resolved_ranges



import pytest
from conan.tools.build import stdcpp_library
from conan.test.utils.mocks import MockSettings, ConanFileMock


@pytest.mark.parametrize("libcxx,expected_library", [
    ("libstdc++", "stdc++"),
    ("libstdc++11", "stdc++"),
    ("libc++", "c++"),
    ("c++_shared", "c++_shared"),
    ("c++_static", "c++_static"),
    ("foobar", None)
])
def test_stdcpp_library(libcxx, expected_library):
    settings = MockSettings({"compiler.libcxx": libcxx})
    conanfile = ConanFileMock(settings)

    assert stdcpp_library(conanfile) == expected_library



import json

import pytest
from conan.test.assets.genconanfile import GenConanfile
from conan.test.utils.tools import TestClient


@pytest.fixture(scope="module")
def build_all():
    """ Build a simple graph to test --build option
        foobar <- bar <- foo
               <--------|
        All packages are built from sources to keep a cache.
    :return: TestClient instance
    """
    client = TestClient()
    client.save({"conanfile.py": GenConanfile().with_setting("build_type")})
    client.run("export . --name=foo --version=1.0 --user=user --channel=testing")
    client.save({"conanfile.py": GenConanfile().with_require("foo/1.0@user/testing")
                .with_setting("build_type")})
    client.run("export . --name=bar --version=1.0 --user=user --channel=testing")
    client.save({"conanfile.py": GenConanfile().with_require("foo/1.0@user/testing")
                .with_require("bar/1.0@user/testing")
                .with_setting("build_type")})
    client.run("export . --name=foobar --version=1.0 --user=user --channel=testing")
    client.run("install --requires=foobar/1.0@user/testing --build='*'")
    return client


foo_id = "efa83b160a55b033c4ea706ddb980cd708e3ba1b"
bar_id = "7d0bb2b97d4339b0d3ded1418a2593f35b9cf267"
foobar_id = "af8f885f621ba7baac3f5b1d2c18cfdf5ba2550c"


def check_if_build_from_sources(refs_modes, output):
    for ref, mode in refs_modes.items():
        if mode == "Build":
            assert "{}/1.0@user/testing: Forced build from source".format(ref) in output
        else:
            assert "{}/1.0@user/testing: Forced build from source".format(ref) not in output


def test_install_build_single(build_all):
    """ When only --build=<ref> is passed, only <ref> must be built
    """
    build_all.run("install --requires=foobar/1.0@user/testing --build=foo/*")
    build_all.assert_listed_binary({"bar/1.0@user/testing": (bar_id, "Cache"),
                                    "foo/1.0@user/testing": (foo_id, "Build"),
                                    "foobar/1.0@user/testing": (foobar_id, "Cache"),
                                    })
    assert "foo/1.0@user/testing: Forced build from source" in build_all.out
    assert "bar/1.0@user/testing: Forced build from source" not in build_all.out
    assert "foobar/1.0@user/testing: Forced build from source" not in build_all.out


def test_install_build_double(build_all):
    """ When both --build=<ref1> and --build=<ref2> are passed, only both should be built
    """
    build_all.run("install --requires=foobar/1.0@user/testing --build=foo/* --build=bar/*")
    build_all.assert_listed_binary({"bar/1.0@user/testing": (bar_id, "Build"),
                                    "foo/1.0@user/testing": (foo_id, "Build"),
                                    "foobar/1.0@user/testing": (foobar_id, "Cache"),
                                    })
    assert "foo/1.0@user/testing: Forced build from source" in build_all.out
    assert "bar/1.0@user/testing: Forced build from source" in build_all.out
    assert "foobar/1.0@user/testing: Forced build from source" not in build_all.out


@pytest.mark.parametrize("build_arg,mode", [("--build=", "Cache"),
                                            ("--build=*", "Build")])
def test_install_build_only(build_arg, mode, build_all):
    """ When only --build is passed wo args, that is a command arg error
        When only --build= is passed, it's a no-op, same as not passing any value
        When only --build=* is passed, all packages must be built from sources
    """
    build_all.run("install --requires=foobar/1.0@user/testing {}".format(build_arg))

    build_all.assert_listed_binary({"bar/1.0@user/testing": (bar_id, mode),
                                    "foo/1.0@user/testing": (foo_id, mode),
                                    "foobar/1.0@user/testing": (foobar_id, mode),
                                    })

    if "Build" == mode:
        assert "foo/1.0@user/testing: Forced build from source" in build_all.out
        assert "bar/1.0@user/testing: Forced build from source" in build_all.out
        assert "foobar/1.0@user/testing: Forced build from source" in build_all.out
    else:
        assert "foo/1.0@user/testing: Forced build from source" not in build_all.out
        assert "bar/1.0@user/testing: Forced build from source" not in build_all.out
        assert "foobar/1.0@user/testing: Forced build from source" not in build_all.out


@pytest.mark.parametrize("build_arg,bar,foo,foobar", [("--build=", "Cache", "Build", "Cache"),
                                                      ("--build=*", "Build", "Build", "Build")])
def test_install_build_all_with_single(build_arg, bar, foo, foobar, build_all):
    """ When --build is passed with another package, only the package must be built from sources.
        When --build= is passed with another package, only the package must be built from sources.
        When --build=* is passed with another package, all packages must be built from sources.
    """
    build_all.run("install --requires=foobar/1.0@user/testing --build=foo/* {}".format(build_arg))
    build_all.assert_listed_binary({"bar/1.0@user/testing": (bar_id, bar),
                                    "foo/1.0@user/testing": (foo_id, foo),
                                    "foobar/1.0@user/testing": (foobar_id, foobar),
                                    })
    check_if_build_from_sources({"foo": foo, "bar": bar, "foobar": foobar}, build_all.out)


@pytest.mark.parametrize("build_arg,bar,foo,foobar", [("--build=", "Cache", "Cache", "Cache"),
                                                      ("--build=*", "Build", "Cache", "Build")])
def test_install_build_all_with_single_skip(build_arg, bar, foo, foobar, build_all):
    """ When --build is passed with a skipped package, not all packages must be built from sources.
        When --build= is passed with another package, only the package must be built from sources.
        When --build=* is passed with another package, not all packages must be built from sources.
        The arguments order matter, that's why we need to run twice.
    """
    for argument in ["--build=!foo/* {}".format(build_arg),
                     "{} --build=!foo/*".format(build_arg)]:
        build_all.run("install --requires=foobar/1.0@user/testing {}".format(argument))
        build_all.assert_listed_binary({"bar/1.0@user/testing": (bar_id, bar),
                                        "foo/1.0@user/testing": (foo_id, foo),
                                        "foobar/1.0@user/testing": (foobar_id, foobar),
                                        })
        check_if_build_from_sources({"foo": foo, "bar": bar, "foobar": foobar}, build_all.out)


@pytest.mark.parametrize("build_arg,bar,foo,foobar", [("--build=", "Cache", "Cache", "Cache"),
                                                      ("--build=*", "Cache", "Cache", "Build")])
def test_install_build_all_with_double_skip(build_arg, bar, foo, foobar, build_all):
    """ When --build is passed with a skipped package, not all packages must be built from sources.
        When --build= is passed with another package, only the package must be built from sources.
        When --build=* is passed with another package, not all packages must be built from sources.
        The arguments order matter, that's why we need to run twice.
    """
    for argument in ["--build=!foo/* --build=~bar/* {}".format(build_arg),
                     "{} --build=!foo/* --build=~bar/*".format(build_arg)]:
        build_all.run("install --requires=foobar/1.0@user/testing {}".format(argument))

        build_all.assert_listed_binary({"bar/1.0@user/testing": (bar_id, bar),
                                        "foo/1.0@user/testing": (foo_id, foo),
                                        "foobar/1.0@user/testing": (foobar_id, foobar),
                                        })


def test_build_consumer():
    """ If the consumer is built from sources, the dependencies must be built too
    """
    client = TestClient()
    client.save({"dep1/conanfile.py": GenConanfile("dep1", "1.0").with_build_msg("TEST: Building dep1"),
                 "dep2/conanfile.py": GenConanfile("dep2", "1.0").with_build_msg("TEST: Building dep2"),
                 "conanfile.py": GenConanfile("consumer", "1.0")
                    .with_build_msg("TEST: Building consumer").with_requires("dep1/1.0", "dep2/1.0")})
    client.run("create dep1")
    client.run("create dep2")
    client.run("export .")


    # Still makes little sense, but just to test the logic
    client.run("create . --build=* --build=!&", assert_error=True)
    assert "TEST: Building consumer" not in client.out

    client.run(f"graph info --requires=consumer/1.0 --build=& -f=json", redirect_stdout="graph.json")
    graph = json.loads(client.load("graph.json"))
    assert graph["graph"]["nodes"]["1"]["binary"] == "Build"  # Would be missing if not built
    assert graph["graph"]["nodes"]["2"]["binary"] == "Cache"
    assert graph["graph"]["nodes"]["3"]["binary"] == "Cache"

    client.run(f"graph info . --build=& -f=json", redirect_stdout="graph.json")
    graph = json.loads(client.load("graph.json"))
    assert graph["graph"]["nodes"]["0"]["binary"] is None
    assert graph["graph"]["nodes"]["1"]["binary"] == "Cache"
    assert graph["graph"]["nodes"]["2"]["binary"] == "Cache"



import os
import platform
import textwrap

import pytest

from conan.test.utils.tools import TestClient


@pytest.mark.skipif(platform.system() != "Darwin", reason="Only OSX")
def test_autotools_universal_binary():
    client = TestClient(path_with_spaces=False)

    conanfile = textwrap.dedent("""
        import os
        from conan import ConanFile
        from conan.tools.gnu import AutotoolsToolchain, Autotools
        from conan.tools.layout import basic_layout

        class mylibraryRecipe(ConanFile):
            name = "mylibrary"
            version = "1.0"
            package_type = "library"
            settings = "os", "compiler", "build_type", "arch"
            options = {"shared": [True, False], "fPIC": [True, False]}
            default_options = {"shared": False, "fPIC": True}

            exports_sources = "configure.ac", "Makefile.am", "src/*"

            def config_options(self):
                if self.settings.os == "Windows":
                    self.options.rm_safe("fPIC")

            def configure(self):
                if self.options.shared:
                    self.options.rm_safe("fPIC")

            def layout(self):
                basic_layout(self)

            def generate(self):
                at_toolchain = AutotoolsToolchain(self)
                at_toolchain.generate()

            def build(self):
                autotools = Autotools(self)
                autotools.autoreconf()
                autotools.configure()
                autotools.make()

            def package(self):
                autotools = Autotools(self)
                autotools.install()
                self.run(f"lipo -info {os.path.join(self.package_folder, 'lib', 'libmylibrary.a')}")

            def package_info(self):
                self.cpp_info.libs = ["mylibrary"]

    """)

    client.run('new autotools_lib -d name=mylibrary -d version=0.1')

    client.save({"conanfile.py": conanfile})

    client.run('create . -s="arch=armv8|x86_64" -tf=""')

    assert "libmylibrary.a are: x86_64 arm64" in client.out



import os
import re
import textwrap

from jinja2 import Template, StrictUndefined

from conan.errors import ConanException
from conan.internal import check_duplicated_generator
from conan.internal.model.dependencies import get_transitive_requires
from conan.internal.util.files import save


class _PCFilesDeps:

    template = textwrap.dedent("""\
        {% for k, v in pc_variables.items() %}
        {{ "{}={}".format(k, v) }}
        {% endfor %}

        Name: {{ name }}
        Description: {{ description }}
        Version: {{ version }}
        {% if libflags %}
        Libs: {{ libflags }}
        {% endif %}
        {% if cflags %}
        Cflags: {{ cflags }}
        {% endif %}
        {% if requires|length %}
        Requires: {{ requires|join(' ') }}
        {% endif %}
    """)

    alias_template = textwrap.dedent("""\
        Name: {{name}}
        Description: Alias {{name}} for {{aliased}}
        Version: {{version}}
        Requires: {{aliased}}
    """)

    def __init__(self, pkgconfigdeps, dep, suffix=""):
        self._conanfile = pkgconfigdeps._conanfile  # noqa
        self._properties = pkgconfigdeps._properties  # noqa
        self._transitive_reqs = get_transitive_requires(self._conanfile, dep)
        self._dep = dep
        self._suffix = suffix

    def _get_aliases(self, dep, pkg_name=None, comp_ref_name=None):
        def _get_dep_aliases():
            pkg_aliases = self._get_property("pkg_config_aliases", dep, check_type=list)
            return pkg_aliases or []

        # TODO: LET'S DEPRECATE ALL THE ALIASES MECHANISM!!
        if pkg_name is None and comp_ref_name is None:
            return _get_dep_aliases()
        if comp_ref_name not in dep.cpp_info.components:
            # Either foo::foo might be referencing the root cpp_info
            if (dep.ref.name == comp_ref_name or
                # Or a "replace_require" is used and cpp_info.requires is the root one, e.g.,
                # zlib/*: zlib-ng/*, and self.cpp_info.requires = ["zlib::zlib"]
                (dep.ref.name != pkg_name and pkg_name == comp_ref_name)):
                return _get_dep_aliases()
            raise ConanException("Component '{name}::{cname}' not found in '{name}' "
                                 "package requirement".format(name=dep.ref.name,
                                                              cname=comp_ref_name))
        comp_aliases = self._get_property("pkg_config_aliases", dep, comp_ref_name, check_type=list)
        return comp_aliases or []

    def _get_name(self, dep, pkg_name=None, comp_ref_name=None):
        def _get_dep_name():
            dep_name = self._get_property("pkg_config_name", dep) or dep.ref.name
            return f"{dep_name}{self._suffix}"

        if pkg_name is None and comp_ref_name is None:
            return _get_dep_name()
        if comp_ref_name not in dep.cpp_info.components:
            # Either foo::foo might be referencing the root cpp_info
            if (dep.ref.name == comp_ref_name or
                # Or a "replace_require" is used and cpp_info.requires is the root one, e.g.,
                # zlib/*: zlib-ng/*, and self.cpp_info.requires = ["zlib::zlib"]
                (dep.ref.name != pkg_name and pkg_name == comp_ref_name)):
                return _get_dep_name()
            raise ConanException("Component '{name}::{cname}' not found in '{name}' "
                                 "package requirement".format(name=dep.ref.name,
                                                              cname=comp_ref_name))
        comp_name = self._get_property("pkg_config_name", dep, comp_ref_name)
        if comp_name:
            return f"{comp_name}{self._suffix}"
        else:
            dep_name = _get_dep_name()
            # Creating a component name with namespace, e.g., dep-comp1
            return f"{dep_name}-{comp_ref_name}"

    def _get_property(self, prop, dep, comp_name=None, check_type=None):
        dep_name = dep.ref.name
        dep_comp = f"{str(dep_name)}::{comp_name}" if comp_name else f"{str(dep_name)}"
        try:
            value = self._properties[f"{dep_comp}{self._suffix}"][prop]
            if check_type is not None and not isinstance(value, check_type):
                raise ConanException(
                    f'The expected type for {prop} is "{check_type.__name__}", but "{type(value).__name__}" was found')
            return value
        except KeyError:
            return dep.cpp_info.get_property(prop, check_type=check_type) if not comp_name \
                else dep.cpp_info.components[comp_name].get_property(prop, check_type=check_type)

    def _get_pc_variables(self, dep, cpp_info, custom_content=None):
        """
        Get all the freeform variables defined by Conan and
        users (through ``pkg_config_custom_content``). This last ones will override the
        Conan defined variables.
        """
        def apply_custom_content():
            if isinstance(custom_content, dict):
                pc_variables.update(custom_content)
            elif custom_content:  # Legacy: custom content is string
                pc_variable_pattern = re.compile("^(.*)=(.*)")
                for line in custom_content.splitlines():
                    match = pc_variable_pattern.match(line)
                    if match:
                        key, value = match.group(1).strip(), match.group(2).strip()
                        pc_variables[key] = value

        # If editable, package_folder can be None
        prefix_path = (dep.recipe_folder if dep.package_folder is None
                                   else dep.package_folder).replace("\\", "/")
        pc_variables = {"prefix": prefix_path}
        # Already formatted directories
        pc_variables.update(self._get_formatted_dirs("libdir", cpp_info.libdirs, prefix_path))
        pc_variables.update(self._get_formatted_dirs("includedir", cpp_info.includedirs, prefix_path))
        pc_variables.update(self._get_formatted_dirs("bindir", cpp_info.bindirs, prefix_path))
        # Get the custom content introduced by user and sanitize it
        apply_custom_content()
        return pc_variables

    @staticmethod
    def _get_formatted_dirs(folder_name, folders, prefix_path_):
        ret = {}
        for i, directory in enumerate(folders):
            directory = os.path.normpath(directory).replace("\\", "/")
            if directory.startswith(prefix_path_):
                prefix = "${prefix}/"
                directory = os.path.relpath(directory, prefix_path_).replace("\\", "/")
            else:
                prefix = "" if os.path.isabs(directory) else "${prefix}/"
            suffix = str(i) if i else ""
            var_name = f"{folder_name}{suffix}"
            ret[var_name] = f"{prefix}{directory}"
        return ret

    def _get_framework_flags(self, cpp_info):
        # FIXME: GnuDepsFlags used only here. Let's adapt the code and remove this dependency.
        #        self._conanfile is also used only here.
        from conan.tools.gnu.gnudeps_flags import GnuDepsFlags
        gnudeps_flags = GnuDepsFlags(self._conanfile, cpp_info)
        return gnudeps_flags.frameworks + gnudeps_flags.framework_paths

    def _get_lib_flags(self, libdirvars, cpp_info):
        framework_flags = self._get_framework_flags(cpp_info)
        libdirsflags = ['-L"${%s}"' % d for d in libdirvars]
        system_libs = ["-l%s" % li for li in (cpp_info.libs + cpp_info.system_libs)]
        shared_flags = cpp_info.sharedlinkflags + cpp_info.exelinkflags
        return " ".join(libdirsflags + system_libs + shared_flags + framework_flags)

    def _get_cflags(self, includedirvars, cpp_info):
        includedirsflags = ['-I"${%s}"' % d for d in includedirvars]
        cxxflags = [var.replace('"', '\\"') for var in cpp_info.cxxflags]
        cflags = [var.replace('"', '\\"') for var in cpp_info.cflags]
        defines = ["-D%s" % var.replace('"', '\\"') for var in cpp_info.defines]
        return " ".join(includedirsflags + cxxflags + cflags + defines)

    def _get_component_requirement_names(self, cpp_info):
        """
        Get all the pkg-config valid names from the requirements ones given a CppInfo object.

        For instance, those requirements could be coming from:

        ```python
        from conan import ConanFile
        class PkgConfigConan(ConanFile):
            requires = "other/1.0"

            def package_info(self):
                self.cpp_info.requires = ["other::cmp1"]

            # Or:

            def package_info(self):
                self.cpp_info.components["cmp"].requires = ["other::cmp1"]
        ```
        """
        dep_ref_name = self._dep.ref.name
        ret = []
        for req in cpp_info.requires:
            pkg_ref_name, comp_ref_name = req.split("::") if "::" in req else (dep_ref_name, req)
            # For instance, dep == "hello/1.0" and req == "other::cmp1" -> hello != other
            if dep_ref_name != pkg_ref_name:
                try:
                    req_conanfile = self._transitive_reqs[pkg_ref_name]
                except KeyError:
                    continue  # If the dependency is not in the transitive, might be skipped
            else:  # For instance, dep == "hello/1.0" and req == "hello::cmp1" -> hello == hello
                req_conanfile = self._dep
            comp_name = self._get_name(req_conanfile, pkg_ref_name, comp_ref_name)
            if comp_name not in ret:
                ret.append(comp_name)
        return ret

    def items(self):
        """
        Get all the PC files and contents for any dependency:

        * If the given dependency does not have components:
            The PC file will be the dependency one.

        * If the given dependency has components:
            The PC files will be saved in this order:
                1- Package components.
                2- Root component.

            Note: If the root-package PC name matches with any other of the components one, the first one
            is not going to be created. Components have more priority than root package.

        * Apart from those PC files, if there are any aliases declared, they will be created too.
        """
        pc_files = {}
        pc_alias_files = {}
        pkg_name = self._get_name(self._dep)
        # First, let's load all the components PC files
        # Loop through all the package's components
        for comp_ref_name, comp_cpp_info in self._dep.cpp_info.get_sorted_components().items():
            # At first, let's check if we have defined some components requires, e.g., "dep::cmp1"
            comp_requires = self._get_component_requirement_names(comp_cpp_info)
            comp_name = self._get_name(self._dep, pkg_name, comp_ref_name)
            version = (self._get_property("component_version", self._dep, comp_ref_name) or
                       self._get_property("system_package_version", self._dep, comp_ref_name) or
                       self._dep.ref.version)
            custom_content = self._get_property("pkg_config_custom_content", self._dep, comp_ref_name)
            pc_variables = self._get_pc_variables(self._dep, comp_cpp_info, custom_content)
            pc_context = {
                "name": comp_name,
                "description": f"Conan component: {comp_name}",
                "version": version,
                "requires": comp_requires,
                "pc_variables": pc_variables,
                "cflags": self._get_cflags([d for d in pc_variables if d.startswith("includedir")],
                                           comp_cpp_info),
                "libflags": self._get_lib_flags([d for d in pc_variables if d.startswith("libdir")],
                                                comp_cpp_info)
            }
            pc_files[comp_name] = self._get_pc_content(pc_context)
            # Aliases
            for alias in self._get_aliases(self._dep, pkg_name, comp_ref_name):
                pc_alias_files[alias] = self._get_alias_pc_content({
                    "name": alias,
                    "version": version,
                    "aliased": comp_name
                })
        # Second, let's load the root package's PC file ONLY
        # if it does not already exist in components one
        # Issue related: https://github.com/conan-io/conan/issues/10341
        should_skip_main = self._get_property("pkg_config_name", self._dep) == "none"
        if pkg_name not in pc_files and not should_skip_main:
            cpp_info = self._dep.cpp_info
            # At first, let's check if we have defined some global requires, e.g., "other::cmp1"
            # Note: If DEP has components, they'll be the requirements == pc_files.keys()
            requires = list(pc_files.keys()) or self._get_component_requirement_names(cpp_info)
            # If we have found some component requirements it would be enough
            if not requires:
                # If no requires were found, let's try to get all the direct visible dependencies,
                # e.g., requires = "other_pkg/1.0"
                requires = [self._get_name(req) for req in self._transitive_reqs.values()]
            version = (self._get_property("system_package_version", self._dep)
                       or self._dep.ref.version)
            custom_content = self._get_property("pkg_config_custom_content", self._dep)
            pc_variables = self._get_pc_variables(self._dep, cpp_info, custom_content)
            pc_context = {
                "name": pkg_name,
                "description": f"Conan package: {pkg_name}",
                "version": version,
                "requires": requires,
                "pc_variables": pc_variables,
                "cflags": self._get_cflags([d for d in pc_variables if d.startswith("includedir")],
                                           cpp_info),
                "libflags": self._get_lib_flags([d for d in pc_variables if d.startswith("libdir")],
                                                cpp_info)
            }
            pc_files[pkg_name] = self._get_pc_content(pc_context)
            # Aliases
            for alias in self._get_aliases(self._dep):
                pc_alias_files[alias] = self._get_alias_pc_content({
                    "name": alias,
                    "version": version,
                    "aliased": pkg_name
                })
        # Adding the aliases
        pc_files.update(pc_alias_files)
        return pc_files.items()

    def _get_pc_content(self, context):
        template = Template(self.template, trim_blocks=True, lstrip_blocks=True,
                            undefined=StrictUndefined)
        return template.render(context)

    def _get_alias_pc_content(self, context):
        template = Template(self.alias_template, trim_blocks=True, lstrip_blocks=True,
                            undefined=StrictUndefined, keep_trailing_newline=True)
        return template.render(context)


class PkgConfigDeps:

    def __init__(self, conanfile):
        self._conanfile = conanfile
        # Activate the build *.pc files for the specified libraries
        self.build_context_activated = []
        # If specified, the files/requires/names for the build context will be renamed appending
        # a suffix. It is necessary in case of same require and build_require and will cause an error
        # DEPRECATED: consumers should use build_context_folder instead
        # FIXME: Conan 3.x: Remove build_context_suffix attribute
        self.build_context_suffix = {}
        # By default, the "[generators_folder]/build" folder will save all the *.pc files activated
        # in the build_context_activated list.
        # Notice that if the `build_context_suffix` attr is defined, the `build_context_folder` one
        # will have no effect.
        # Issue: https://github.com/conan-io/conan/issues/12342
        # Issue: https://github.com/conan-io/conan/issues/14935
        # FIXME: Conan 3.x: build_context_folder should be "build" by default
        self.build_context_folder = None  # Keeping backward-compatibility
        self._properties = {}

    def _get_dependencies(self):
        # Get all the dependencies
        host_req = self._conanfile.dependencies.host
        build_req = self._conanfile.dependencies.build  # tool_requires
        test_req = self._conanfile.dependencies.test
        # If self.build_context_suffix is not defined, the build requires will be saved
        # in the self.build_context_folder
        # FIXME: Conan 3.x: Remove build_context_suffix attribute and the validation function
        if self.build_context_folder is None:  # Legacy flow
            if self.build_context_suffix:
                # deprecation warning
                self._conanfile.output.warning(
                    "PkgConfigDeps.build_context_suffix attribute has been "
                    "deprecated. Use PkgConfigDeps.build_context_folder instead."
                )
            # Check if it exists both as require and as build require without a suffix
            activated_br = {r.ref.name for r in build_req.values()
                            if r.ref.name in self.build_context_activated}
            common_names = {r.ref.name for r in host_req.values()}.intersection(activated_br)
            without_suffixes = [common_name for common_name in common_names
                                if not self.build_context_suffix.get(common_name)]
            if without_suffixes:
                raise ConanException(
                    f"The packages {without_suffixes} exist both as 'require' and as"
                    f" 'build require'. You need to specify a suffix using the "
                    f"'build_context_suffix' attribute at the PkgConfigDeps generator.")
        elif self.build_context_folder is not None and self.build_context_suffix:
            raise ConanException(
                "It's not allowed to define both PkgConfigDeps.build_context_folder "
                "and PkgConfigDeps.build_context_suffix (deprecated).")

        for require, dep in list(host_req.items()) + list(build_req.items()) + list(test_req.items()):
            # Filter the build_requires not activated with PkgConfigDeps.build_context_activated
            if require.build and dep.ref.name not in self.build_context_activated:
                continue
            yield require, dep

    def generate(self):
        """
        Save all the `*.pc` files
        """
        def _pc_file_name(name_, is_build_context=False, has_suffix=False):
            # If no suffix is defined, we can save the *.pc file in the build_context_folder
            build = is_build_context and self.build_context_folder and not has_suffix
            # Issue: https://github.com/conan-io/conan/issues/12342
            # Issue: https://github.com/conan-io/conan/issues/14935
            return f"{self.build_context_folder}/{name_}.pc" if build else f"{name_}.pc"

        check_duplicated_generator(self, self._conanfile)
        for require, dep in self._get_dependencies():
            suffix = self.build_context_suffix.get(require.ref.name, "") if require.build else ""
            # Save all the *.pc files and their contents
            for name, content in _PCFilesDeps(self, dep, suffix=suffix).items():
                pc_name = _pc_file_name(name, is_build_context=require.build,
                                        has_suffix=bool(suffix))
                save(pc_name, content)

    def set_property(self, dep, prop, value):
        """
        Using this method you can overwrite the :ref:`property<PkgConfigDeps Properties>` values set by
        the Conan recipes from the consumer. This can be done for `pkg_config_name`,
        `pkg_config_aliases` and `pkg_config_custom_content` properties.

        :param dep: Name of the dependency to set the :ref:`property<PkgConfigDeps Properties>`. For
         components use the syntax: ``dep_name::component_name``.
        :param prop: Name of the :ref:`property<PkgConfigDeps Properties>`.
        :param value: Value of the property. Use ``None`` to invalidate any value set by the
         upstream recipe.
        """
        self._properties.setdefault(dep, {}).update({prop: value})



import os
import platform
import re
import textwrap

import pytest

from conan.test.assets.genconanfile import GenConanfile
from conan.test.utils.tools import TestClient
from conan.tools.meson import MesonToolchain


def test_apple_meson_keep_user_custom_flags():
    default = textwrap.dedent("""
    [settings]
    os=Macos
    arch=x86_64
    compiler=apple-clang
    compiler.version=12.0
    compiler.libcxx=libc++
    build_type=Release
    """)

    cross = textwrap.dedent("""
    [settings]
    os = iOS
    os.version = 10.0
    os.sdk = iphoneos
    arch = armv8
    compiler = apple-clang
    compiler.version = 12.0
    compiler.libcxx = libc++

    [conf]
    tools.apple:sdk_path=/my/sdk/path
    """)

    _conanfile_py = textwrap.dedent("""
    from conan import ConanFile
    from conan.tools.meson import MesonToolchain

    class App(ConanFile):
        settings = "os", "arch", "compiler", "build_type"

        def generate(self):
            tc = MesonToolchain(self)
            # Customized apple flags
            tc.apple_arch_flag = ['-arch', 'myarch']
            tc.apple_isysroot_flag = ['-isysroot', '/other/sdk/path']
            tc.apple_min_version_flag = ['-otherminversion=10.7']
            tc.generate()
    """)

    t = TestClient()
    t.save({"conanfile.py": _conanfile_py,
            "build_prof": default,
            "host_prof": cross})

    t.run("install . -pr:h host_prof -pr:b build_prof")
    # Checking that the global conanbuild aggregator includes conanbuildenv-xxx file
    # it should have been generated by implicit VirtualBuildEnv generator
    env_file = t.load("conanbuild.sh")
    assert "conanbuildenv" in env_file
    content = t.load(MesonToolchain.cross_filename)
    assert "c_args = ['-isysroot', '/other/sdk/path', '-arch', 'myarch', '-otherminversion=10.7']" in content
    assert "c_link_args = ['-isysroot', '/other/sdk/path', '-arch', 'myarch', '-otherminversion=10.7']" in content
    assert "cpp_args = ['-isysroot', '/other/sdk/path', '-arch', 'myarch', '-otherminversion=10.7', '-stdlib=libc++']" in content
    assert "cpp_link_args = ['-isysroot', '/other/sdk/path', '-arch', 'myarch', '-otherminversion=10.7', '-stdlib=libc++']" in content


def test_apple_meson_cross_building_subsystem():
    """
    Issue related: https://github.com/conan-io/conan/issues/17873
    """
    default = textwrap.dedent("""
    [settings]
    os=Macos
    arch=x86_64
    compiler=apple-clang
    compiler.version=12.0
    compiler.libcxx=libc++
    build_type=Release
    """)
    cross = textwrap.dedent("""
    [settings]
    os = iOS
    os.version = 10.0
    os.sdk = iphoneos
    arch = armv8
    compiler = apple-clang
    compiler.version = 12.0
    compiler.libcxx = libc++

    [conf]
    tools.apple:sdk_path=/my/sdk/path
    """)
    t = TestClient()
    t.save({"conanfile.py": GenConanfile(name="app", version="1.0")
                            .with_settings("os", "arch", "compiler", "build_type")
                            .with_generator("MesonToolchain"),
            "build": default,
            "host": cross})
    t.run("install . -pr:h host -pr:b build")
    content = t.load(MesonToolchain.cross_filename)
    machines_settings = textwrap.dedent("""\
    [build_machine]
    system = 'darwin'
    subsystem = 'macos'
    cpu_family = 'x86_64'
    cpu = 'x86_64'
    endian = 'little'
    [host_machine]
    system = 'darwin'
    subsystem = 'ios'
    cpu_family = 'aarch64'
    cpu = 'armv8'
    endian = 'little'""")
    assert machines_settings in content
    # Let's check that it does not appear if cross-compiling to other non-Apple-OS
    cross = textwrap.dedent("""
    [settings]
    arch=armv8
    build_type=Release
    compiler=gcc
    compiler.cppstd=gnu17
    compiler.libcxx=libstdc++11
    compiler.version=13
    os=Linux
    """)
    t.save({"host": cross})
    t.run("install . -pr:h host -pr:b build")
    content = t.load(MesonToolchain.cross_filename)
    assert "subsystem =" not in content


def test_extra_flags_via_conf():
    profile = textwrap.dedent("""
        [settings]
        os=Windows
        arch=x86_64
        compiler=gcc
        compiler.version=9
        compiler.cppstd=17
        compiler.libcxx=libstdc++
        build_type=Release

        [buildenv]
        CFLAGS=-flag0 -other=val
        CXXFLAGS=-flag0 -other=val
        LDFLAGS=-flag0 -other=val

        [conf]
        tools.build:cxxflags=["-flag1", "-flag2"]
        tools.build:cflags=["-flag3", "-flag4"]
        tools.build:sharedlinkflags+=["-flag5"]
        tools.build:exelinkflags+=["-flag6"]
        # Issue related: https://github.com/conan-io/conan/issues/16169
        tools.build:defines=["define1=0"]
   """)
    t = TestClient()
    t.save({"conanfile.txt": "[generators]\nMesonToolchain",
            "profile": profile})

    t.run("install . -pr:h=profile -pr:b=profile")
    content = t.load(MesonToolchain.native_filename)
    assert "cpp_args = ['-flag0', '-other=val', '-m64', '-flag1', '-flag2', '-Ddefine1=0', '-D_GLIBCXX_USE_CXX11_ABI=0']" in content
    assert "c_args = ['-flag0', '-other=val', '-m64', '-flag3', '-flag4', '-Ddefine1=0']" in content
    assert "c_link_args = ['-flag0', '-other=val', '-m64', '-flag5', '-flag6']" in content
    assert "cpp_link_args = ['-flag0', '-other=val', '-m64', '-flag5', '-flag6']" in content


def test_extra_flags_via_toolchain():
    profile = textwrap.dedent("""
        [settings]
        os=Windows
        arch=x86_64
        compiler=gcc
        compiler.version=9
        compiler.cppstd=17
        compiler.libcxx=libstdc++
        build_type=Release

        [buildenv]
        CFLAGS=-flag0 -other=val
        CXXFLAGS=-flag0 -other=val
        LDFLAGS=-flag0 -other=val
   """)
    t = TestClient()
    conanfile = textwrap.dedent("""
    from conan import ConanFile
    from conan.tools.meson import MesonToolchain
    class Pkg(ConanFile):
        settings = "os", "compiler", "arch", "build_type"
        def generate(self):
            tc = MesonToolchain(self)
            tc.extra_cxxflags = ["-flag1", "-flag2"]
            tc.extra_cflags = ["-flag3", "-flag4"]
            tc.extra_ldflags = ["-flag5", "-flag6"]
            tc.extra_defines = ["define1=0"]
            tc.generate()
    """)
    t.save({"conanfile.py": conanfile,
            "profile": profile})
    t.run("install . -pr:h=profile -pr:b=profile")
    content = t.load(MesonToolchain.native_filename)
    assert "cpp_args = ['-flag0', '-other=val', '-m64', '-flag1', '-flag2', '-Ddefine1=0', '-D_GLIBCXX_USE_CXX11_ABI=0']" in content
    assert "c_args = ['-flag0', '-other=val', '-m64', '-flag3', '-flag4', '-Ddefine1=0']" in content
    assert "c_link_args = ['-flag0', '-other=val', '-m64', '-flag5', '-flag6']" in content
    assert "cpp_link_args = ['-flag0', '-other=val', '-m64', '-flag5', '-flag6']" in content


def test_custom_arch_flag_via_toolchain():
    t = TestClient()
    conanfile = textwrap.dedent("""
    from conan import ConanFile
    from conan.tools.meson import MesonToolchain
    class Pkg(ConanFile):
        settings = "os", "compiler", "arch", "build_type"
        def generate(self):
            tc = MesonToolchain(self)
            tc.arch_flag = "-mmy-flag"
            tc.generate()
    """)
    t.save({"conanfile.py": conanfile})
    t.run("install .")
    content = t.load(MesonToolchain.native_filename)
    assert re.search(r"c_args =.+-mmy-flag.+", content)
    assert re.search(r"c_link_args =.+-mmy-flag.+", content)
    assert re.search(r"cpp_args =.+-mmy-flag.+", content)
    assert re.search(r"cpp_link_args =.+-mmy-flag.+", content)


def test_linker_scripts_via_conf():
    profile = textwrap.dedent("""
        [settings]
        os=Windows
        arch=x86_64
        compiler=gcc
        compiler.version=9
        compiler.cppstd=17
        compiler.libcxx=libstdc++
        build_type=Release

        [buildenv]
        LDFLAGS=-flag0 -other=val

        [conf]
        tools.build:sharedlinkflags+=["-flag5"]
        tools.build:exelinkflags+=["-flag6"]
        tools.build:linker_scripts+=["/linker/scripts/flash.ld", "/linker/scripts/extra_data.ld"]
   """)
    t = TestClient()
    t.save({"conanfile.txt": "[generators]\nMesonToolchain",
            "profile": profile})

    t.run("install . -pr:b=profile -pr=profile")
    content = t.load(MesonToolchain.native_filename)
    assert ("c_link_args = ['-flag0', '-other=val', '-m64', '-flag5', '-flag6', "
            "'-T/linker/scripts/flash.ld', '-T/linker/scripts/extra_data.ld']") in content
    assert ("cpp_link_args = ['-flag0', '-other=val', '-m64', '-flag5', '-flag6', "
            "'-T/linker/scripts/flash.ld', '-T/linker/scripts/extra_data.ld']") in content


def test_correct_quotes():
    profile = textwrap.dedent("""
       [settings]
       os=Windows
       arch=x86_64
       compiler=gcc
       compiler.version=9
       compiler.cppstd=17
       compiler.libcxx=libstdc++11
       build_type=Release
       """)
    t = TestClient()
    t.save({"conanfile.txt": "[generators]\nMesonToolchain",
            "profile": profile})

    t.run("install . -pr:h=profile -pr:b=profile")
    content = t.load(MesonToolchain.native_filename)
    assert "cpp_std = 'c++17'" in content
    assert "backend = 'ninja'" in content
    assert "buildtype = 'release'" in content


def test_c_std():
    profile = textwrap.dedent("""
       [settings]
       os=Windows
       arch=x86_64
       compiler=gcc
       compiler.version=9
       compiler.cstd=11
       build_type=Release
       """)
    t = TestClient()
    t.save({"conanfile.py": GenConanfile().with_settings("os", "compiler", "build_type", "arch")
                                          .with_generator("MesonToolchain")
                                          .with_class_attribute("languages='C'"),
            "profile": profile})

    t.run("install . -pr:h=profile -pr:b=profile")
    content = t.load(MesonToolchain.native_filename)
    assert "c_std = 'c11'" in content
    assert "backend = 'ninja'" in content
    assert "buildtype = 'release'" in content


def test_deactivate_nowrap():
    # https://github.com/conan-io/conan/issues/10671
    t = TestClient()
    conanfile = textwrap.dedent("""
        from conan import ConanFile
        from conan.tools.meson import MesonToolchain
        class Pkg(ConanFile):
            settings = "os", "compiler", "arch", "build_type"
            def generate(self):
                tc = MesonToolchain(self)
                tc.project_options.pop("wrap_mode")
                tc.generate()
        """)
    t.save({"conanfile.py": conanfile})
    t.run("install .")
    content = t.load(MesonToolchain.native_filename)
    assert "wrap_mode " not in content
    assert "nofallback" not in content


@pytest.mark.skipif(platform.system() != "Windows", reason="requires Win")
@pytest.mark.parametrize("build_type,runtime,vscrt", [
    ("Debug", "dynamic", "mdd"),
    ("Debug", "static", "mtd"),
    ("Release", "dynamic", "md"),
    ("Release", "static", "mt")
])
def test_clang_cl_vscrt(build_type, runtime, vscrt):
    profile = textwrap.dedent(f"""
        [settings]
        os=Windows
        arch=x86_64
        build_type={build_type}
        compiler=clang
        compiler.runtime={runtime}
        compiler.runtime_version=v143
        compiler.version=16

        [conf]
        tools.cmake.cmaketoolchain:generator=Visual Studio 17

        [buildenv]
        CC=clang-cl
        CXX=clang-cl
   """)
    t = TestClient()
    t.save({"conanfile.txt": "[generators]\nMesonToolchain",
            "profile": profile})

    t.run("install . -pr:h=profile -pr:b=profile")
    content = t.load(MesonToolchain.native_filename)
    assert f"b_vscrt = '{vscrt}'" in content


def test_env_vars_from_build_require():
    conanfile = textwrap.dedent("""
    from conan import ConanFile
    import os

    class HelloConan(ConanFile):
        name = 'hello_compiler'
        version = '1.0'
        def package_info(self):
            self.buildenv_info.define("CC", "CC_VALUE")
            self.buildenv_info.define("CC_LD", "CC_LD_VALUE")
            self.buildenv_info.define("CXX", "CXX_VALUE")
            self.buildenv_info.define("CXX_LD", "CXX_LD_VALUE")
            self.buildenv_info.define("AR", "AR_VALUE")
            self.buildenv_info.define("STRIP", "STRIP_VALUE")
            self.buildenv_info.define("AS", "AS_VALUE")
            self.buildenv_info.define("WINDRES", "WINDRES_VALUE")
            self.buildenv_info.define("PKG_CONFIG", "PKG_CONFIG_VALUE")
            self.buildenv_info.define("LD", "LD_VALUE")
    """)
    client = TestClient()
    client.save({"conanfile.py": conanfile})
    client.run("create .")

    conanfile = textwrap.dedent("""
    from conan import ConanFile
    class HelloConan(ConanFile):
        name = 'consumer'
        version = '1.0'
        generators = "MesonToolchain"
        settings = "os", "arch", "compiler", "build_type"

        def requirements(self):
            self.tool_requires("hello_compiler/1.0", )
    """)
    # Now, let's check how all the build env variables are applied at consumer side
    client.save({"conanfile.py": conanfile})
    client.run("install . -pr:h=default -pr:b=default")
    content = client.load("conan_meson_native.ini")
    assert "c = 'CC_VALUE'" in content
    assert "cpp = 'CXX_VALUE'" in content
    assert "ld = 'LD_VALUE'" in content
    assert "c_ld = 'CC_LD_VALUE'" in content
    assert "cpp_ld = 'CXX_LD_VALUE'" in content
    assert "ar = 'AR_VALUE'" in content
    assert "strip = 'STRIP_VALUE'" in content
    assert "as = 'AS_VALUE'" in content
    assert "windres = 'WINDRES_VALUE'" in content
    assert "pkgconfig = 'PKG_CONFIG_VALUE'" in content
    assert "pkg-config = 'PKG_CONFIG_VALUE'" in content


def test_check_c_cpp_ld_list_formats():
    # Issue related: https://github.com/conan-io/conan/issues/14028
    profile = textwrap.dedent("""
       [settings]
       os=Windows
       arch=x86_64
       compiler=gcc
       compiler.version=9
       compiler.cppstd=17
       compiler.libcxx=libstdc++11
       build_type=Release
       [buildenv]
       CC=aarch64-poky-linux-gcc  -mcpu=cortex-a53 -march=armv8-a+crc+crypto
       CXX=aarch64-poky-linux-g++  -mcpu=cortex-a53 -march=armv8-a+crc+crypto
       LD=aarch64-poky-linux-ld  --sysroot=/opt/sysroots/cortexa53-crypto-poky-linux
       """)
    t = TestClient()
    t.save({"conanfile.txt": "[generators]\nMesonToolchain",
            "profile": profile})
    t.run("install . -pr:h=profile -pr:b=profile")
    content = t.load(MesonToolchain.native_filename)
    assert "c = ['aarch64-poky-linux-gcc', '-mcpu=cortex-a53', '-march=armv8-a+crc+crypto']" in content
    assert "cpp = ['aarch64-poky-linux-g++', '-mcpu=cortex-a53', '-march=armv8-a+crc+crypto']" in content
    assert "ld = ['aarch64-poky-linux-ld', '--sysroot=/opt/sysroots/cortexa53-crypto-poky-linux']" in content


def test_check_pkg_config_paths():
    # Issue: https://github.com/conan-io/conan/issues/12342
    # Issue: https://github.com/conan-io/conan/issues/14935
    t = TestClient()
    t.save({"conanfile.txt": "[generators]\nMesonToolchain"})
    t.run("install .")
    content = t.load(MesonToolchain.native_filename)
    assert f"pkg_config_path = '{t.current_folder}'" in content
    assert f"build.pkg_config_path = " not in content
    conanfile = textwrap.dedent("""
    import os
    from conan import ConanFile
    from conan.tools.meson import MesonToolchain
    class Pkg(ConanFile):
        settings = "os", "compiler", "arch", "build_type"
        def generate(self):
            tc = MesonToolchain(self)
            tc.build_pkg_config_path = os.path.join(self.generators_folder, "build")
            tc.generate()
    """)
    t.save({"conanfile.py": conanfile}, clean_first=True)
    t.run("install .")
    content = t.load(MesonToolchain.native_filename)
    base_folder = t.current_folder
    assert f"pkg_config_path = '{base_folder}'" in content
    assert f"build.pkg_config_path = '{os.path.join(base_folder, 'build')}'" in content


def test_toolchain_and_compilers_build_context():
    """
    Tests how MesonToolchain manages the build context profile if the build profile is
    specifying another compiler path (using conf).

    It should create both native and cross files.

    Issue related: https://github.com/conan-io/conan/issues/15878
    """
    host = textwrap.dedent("""
    [settings]
    arch=armv8
    build_type=Release
    compiler=gcc
    compiler.cppstd=gnu17
    compiler.libcxx=libstdc++11
    compiler.version=11
    os=Linux

    [conf]
    tools.build:compiler_executables={"c": "gcc", "cpp": "g++"}
    """)
    build = textwrap.dedent("""
    [settings]
    os=Linux
    arch=x86_64
    compiler=clang
    compiler.version=12
    compiler.libcxx=libc++
    compiler.cppstd=11

    [conf]
    tools.build:compiler_executables={"c": "clang", "cpp": "clang++"}
    """)
    tool = textwrap.dedent("""
    import os
    from conan import ConanFile
    from conan.tools.files import load

    class toolRecipe(ConanFile):
        name = "tool"
        version = "1.0"
        # Binary configuration
        settings = "os", "compiler", "build_type", "arch"
        generators = "MesonToolchain"

        def build(self):
            toolchain = os.path.join(self.generators_folder, "conan_meson_native.ini")
            content = load(self, toolchain)
            assert "c = 'clang'" in content
            assert "cpp = 'clang++'" in content
    """)
    consumer = textwrap.dedent("""
    import os
    from conan import ConanFile
    from conan.tools.files import load

    class consumerRecipe(ConanFile):
        name = "consumer"
        version = "1.0"
        # Binary configuration
        settings = "os", "compiler", "build_type", "arch"
        generators = "MesonToolchain"
        tool_requires = "tool/1.0"

        def build(self):
            toolchain = os.path.join(self.generators_folder, "conan_meson_cross.ini")
            content = load(self, toolchain)
            assert "c = 'gcc'" in content
            assert "cpp = 'g++'" in content
    """)
    client = TestClient()
    client.save({
        "host": host,
        "build": build,
        "tool/conanfile.py": tool,
        "consumer/conanfile.py": consumer
    })
    client.run("export tool")
    client.run("create consumer -pr:h host -pr:b build --build=missing")


def test_subproject_options():
    t = TestClient()
    conanfile = textwrap.dedent("""
        from conan import ConanFile
        from conan.tools.meson import MesonToolchain
        class Pkg(ConanFile):
            settings = "os", "compiler", "arch", "build_type"
            def generate(self):
                tc = MesonToolchain(self)
                tc.subproject_options["subproject1"] = [{"option1": "enabled"}, {"option2": "disabled"}]
                tc.subproject_options["subproject2"] = [{"option3": "enabled"}]
                tc.subproject_options["subproject2"].append({"option4": "disabled"})
                tc.generate()
        """)
    t.save({"conanfile.py": conanfile})
    t.run("install .")
    content = t.load(MesonToolchain.native_filename)
    assert "[subproject1:project options]" in content
    assert "[subproject2:project options]" in content
    assert "option1 = 'enabled'" in content
    assert "option2 = 'disabled'" in content
    assert "option3 = 'enabled'" in content
    assert "option4 = 'disabled'" in content


def test_native_attribute():
    """
    Tests that native file only has the build context (not as a build require) content.
    """
    host = textwrap.dedent("""
    [settings]
    arch=armv8
    build_type=Release
    compiler=gcc
    compiler.cppstd=gnu17
    compiler.libcxx=libstdc++11
    compiler.version=11
    os=Linux

    [buildenv]
    STRIP=False
    PKG_CONFIG=/usr/bin/pkg-config

    [conf]
    tools.build:compiler_executables={"c": "gcc", "cpp": "g++"}
    """)
    build = textwrap.dedent("""
    [settings]
    os=Linux
    arch=x86_64
    compiler=clang
    compiler.version=12
    compiler.libcxx=libc++
    compiler.cppstd=11

    [buildenv]
    STRIP=True
    PKG_CONFIG=/usr/lib/meson/pkgconfig

    [conf]
    tools.build:compiler_executables={"c": "clang", "cpp": "clang++"}
    """)
    client = TestClient()
    conanfile = textwrap.dedent("""
    from conan import ConanFile
    from conan.tools.meson import MesonToolchain
    class Pkg(ConanFile):
        settings = "os", "compiler", "arch", "build_type"
        def generate(self):
            tc = MesonToolchain(self)
            tc.generate()
            # We're cross-building, no need to check it
            tc = MesonToolchain(self, native=True)
            tc.generate()
    """)
    client.save({"host": host,
                 "build": build,
                 "conanfile.py": conanfile})
    client.run("install . -pr:h host -pr:b build")
    native_content = client.load(MesonToolchain.native_filename)
    cross_content = client.load(MesonToolchain.cross_filename)
    expected_native = textwrap.dedent("""
    [binaries]
    c = 'clang'
    cpp = 'clang++'
    strip = 'True'
    pkgconfig = '/usr/lib/meson/pkgconfig'
    pkg-config = '/usr/lib/meson/pkgconfig'
    """)
    expected_cross = textwrap.dedent("""
    [binaries]
    c = 'gcc'
    cpp = 'g++'
    strip = 'False'
    pkgconfig = '/usr/bin/pkg-config'
    pkg-config = '/usr/bin/pkg-config'
    """)
    assert expected_native in native_content
    assert "[host_machine]" not in native_content
    assert "[build_machine]" not in native_content
    assert expected_cross in cross_content
    assert "[build_machine]" in cross_content
    assert "[host_machine]" in cross_content


def test_native_attribute_error():
    client = TestClient()
    conanfile = textwrap.dedent("""
    from conan import ConanFile
    from conan.tools.meson import MesonToolchain
    class Pkg(ConanFile):
        settings = "os", "compiler", "arch", "build_type"
        def generate(self):
            tc = MesonToolchain(self)
            tc.generate()
            tc = MesonToolchain(self, native=True)
            tc.generate()
    """)
    client.save({"conanfile.py": conanfile})
    client.run("install .", assert_error=True)
    assert "You can only pass native=True if you're cross-building" in client.out


def test_compiler_path_with_spaces():
    profile = textwrap.dedent("""
        include(default)
        [conf]
        tools.build:compiler_executables={"c":"c compiler path with spaces", "cpp":"cpp compiler path with spaces"}
    """)
    client = TestClient()
    client.save(
        {"conanfile.py": GenConanfile().with_generator("MesonToolchain").with_settings("compiler"),
         "meson-profile": profile})
    client.run("install . -pr:h=meson-profile")
    conan_meson_native = client.load("conan_meson_native.ini")
    assert "c = 'c compiler path with spaces'" in conan_meson_native
    assert "cpp = 'cpp compiler path with spaces'" in conan_meson_native


def test_meson_sysroot_app():
    """Testing when users pass tools.build:sysroot on the profile with Meson

    * The generated conan_meson_cross.ini does not fill the "sys_root" property (see https://github.com/conan-io/conan/issues/16468)
    * It adds the compiler flags with --sysroot.

    When cross-building, Meson needs both compiler_executables in the config, otherwise it will fail
    when running setup.
    """
    sysroot = "/my/new/sysroot/path"
    client = TestClient()
    profile = textwrap.dedent(f"""
    [settings]
    os = Macos
    arch = armv8
    compiler = apple-clang
    compiler.version = 13.0
    compiler.libcxx = libc++

    [conf]
    tools.build:sysroot={sysroot}
    tools.build:verbosity=verbose
    tools.compilation:verbosity=verbose
    tools.apple:sdk_path=/my/sdk/path
    """)
    profile_build = textwrap.dedent(f"""
    [settings]
    os = Macos
    arch = x86_64
    compiler = apple-clang
    compiler.version = 13.0
    compiler.libcxx = libc++
    """)
    client.save({"conanfile.py": GenConanfile(name="hello", version="0.1")
                .with_settings("os", "arch", "compiler", "build_type")
                .with_generator("MesonToolchain"),
                 "build": profile_build,
                 "host": profile})
    client.run("install . -pr:h host -pr:b build")
    # Check the meson configuration file
    conan_meson = client.load("conan_meson_cross.ini")
    assert f"sys_root = '{sysroot}'\n" not in conan_meson
    assert re.search(r"c_args =.+--sysroot={}.+".format(sysroot), conan_meson)
    assert re.search(r"c_link_args =.+--sysroot={}.+".format(sysroot), conan_meson)
    assert re.search(r"cpp_args =.+--sysroot={}.+".format(sysroot), conan_meson)
    assert re.search(r"cpp_link_args =.+--sysroot={}.+".format(sysroot), conan_meson)


def test_cross_x86_64_to_x86():
    """
    https://github.com/conan-io/conan/issues/17261
    """

    c = TestClient()
    c.save({"conanfile.py": GenConanfile().with_settings("os", "compiler", "arch", "build_type")})
    c.run("install . -g MesonToolchain -s arch=x86 -s:b arch=x86_64")
    assert not os.path.exists(os.path.join(c.current_folder, MesonToolchain.native_filename))
    cross = c.load(MesonToolchain.cross_filename)
    assert "cpu = 'x86_64'" in cross  # This is the build machine
    assert "cpu = 'x86'" in cross  # This is the host machine


def test_cross_x86_64_to_riscv32():
    """
    https://github.com/conan-io/conan/issues/18490
    """
    c = TestClient()
    c.save({"conanfile.py": GenConanfile().with_settings("os", "compiler", "arch", "build_type")})
    c.run("install . -g MesonToolchain -s os=Linux -s arch=riscv32 -s:b arch=x86_64")
    assert not os.path.exists(os.path.join(c.current_folder, MesonToolchain.native_filename))
    cross = c.load(MesonToolchain.cross_filename)
    assert "cpu = 'x86_64'" in cross  # This is the build machine
    assert "cpu = 'riscv32'" in cross  # This is the host machine


def test_cross_x86_64_to_riscv64():
    """
    https://github.com/conan-io/conan/issues/18490
    """
    c = TestClient()
    c.save({"conanfile.py": GenConanfile().with_settings("os", "compiler", "arch", "build_type")})
    c.run("install . -g MesonToolchain -s os=Linux -s arch=riscv64 -s:b arch=x86_64")
    assert not os.path.exists(os.path.join(c.current_folder, MesonToolchain.native_filename))
    cross = c.load(MesonToolchain.cross_filename)
    assert "cpu = 'x86_64'" in cross  # This is the build machine
    assert "cpu = 'riscv64'" in cross  # This is the host machine


def test_conf_extra_apple_flags():
    host = textwrap.dedent("""
    [settings]
    arch=x86_64
    os=Macos
    compiler=apple-clang
    compiler.version=14.0
    [conf]
    tools.apple:enable_bitcode = True
    tools.apple:enable_arc = True
    tools.apple:enable_visibility = True
    """)

    c = TestClient()
    c.save({"conanfile.txt": f"[generators]\nMesonToolchain",
            "host": host})
    c.run("install . -pr:a host")
    f = "conan_meson_native.ini"
    tc = c.load(f)
    for flags in ["c_args", "cpp_args", "c_link_args", "cpp_link_args"]:
        assert f"{flags} = ['-m64', '-fembed-bitcode', '-fvisibility=default']" in tc
    for flags in ["objcpp_args", "objc_args"]:
        assert f"{flags} = ['-fobjc-arc', '-m64', '-fembed-bitcode', '-fvisibility=default']" in tc
    c.run("install . -pr:a host -s build_type=Debug")
    tc = c.load(f)
    for flags in ["c_args", "cpp_args", "c_link_args", "cpp_link_args"]:
        assert f"{flags} = ['-m64', '-fembed-bitcode-marker', '-fvisibility=default']" in tc
    for flags in ["objcpp_args", "objc_args"]:
        assert f"{flags} = ['-fobjc-arc', '-m64', '-fembed-bitcode-marker', '-fvisibility=default']" in tc

    host = textwrap.dedent("""
        [settings]
        arch=x86_64
        os=Macos
        compiler=apple-clang
        compiler.version=14.0
        [conf]
        tools.apple:enable_bitcode = False
        tools.apple:enable_arc = False
        tools.apple:enable_visibility = False
        """)

    c.save({"host": host})
    c.run("install . -pr:a host")
    tc = c.load(f)
    for flags in ["c_args", "cpp_args", "c_link_args", "cpp_link_args"]:
        assert f"{flags} = ['-m64', '-fvisibility=hidden', '-fvisibility-inlines-hidden']" in tc
    for flags in ["objcpp_args", "objc_args"]:
        assert f"{flags} = ['-fno-objc-arc', '-m64', '-fvisibility=hidden', '-fvisibility-inlines-hidden']" in tc

@pytest.mark.parametrize(
    "threads, flags",
    [("posix", "-pthread"), ("wasm_workers", "-sWASM_WORKERS=1")],
)
def test_thread_flags(threads, flags):
    client = TestClient()
    profile = textwrap.dedent(f"""
        [settings]
        arch=wasm
        build_type=Release
        compiler=emcc
        compiler.cppstd=17
        compiler.threads={threads}
        compiler.libcxx=libc++
        compiler.version=4.0.10
        os=Emscripten
        """)
    client.save(
        {
            "conanfile.py": GenConanfile("pkg", "1.0")
            .with_settings("os", "arch", "compiler", "build_type")
            .with_generator("MesonToolchain"),
            "profile": profile,
        }
    )
    client.run("install . -pr=./profile")
    toolchain = client.load("conan_meson_cross.ini")
    assert f"c_args = ['{flags}']" in toolchain
    assert f"c_link_args = ['{flags}']" in toolchain
    assert f"cpp_args = ['{flags}', '-stdlib=libc++']" in toolchain
    assert f"cpp_link_args = ['{flags}', '-stdlib=libc++']" in toolchain


def test_new_public_attributes():
    host = textwrap.dedent(f"""
    [settings]
    arch=armv8
    build_type=Release
    compiler=gcc
    compiler.cppstd=gnu17
    compiler.libcxx=libstdc++11
    compiler.version=11
    os=Linux
    [conf]
    tools.meson.mesontoolchain:backend=xcode
    """)
    client = TestClient()
    conanfile = textwrap.dedent("""
    from conan import ConanFile
    from conan.tools.meson import MesonToolchain
    class Pkg(ConanFile):
        settings = "os", "compiler", "arch", "build_type"
        def generate(self):
            tc = MesonToolchain(self)
            tc.backend = "vs2022"  # conf has more prio
            tc.b_staticpic = True
            tc.buildtype = "Debug"
            tc.default_library = "shared"
            tc.cpp_std="c++20"
            tc.c_std="c20"
            tc.b_vscrt="MD"
            tc.generate()
    """)
    client.save({"conanfile.py": conanfile,
                 "host": host})
    client.run("install . -pr:a host")
    content = client.load(MesonToolchain.native_filename)
    expected = textwrap.dedent("""\
    buildtype = 'Debug'
    default_library = 'shared'
    b_vscrt = 'MD'
    b_ndebug = 'true'
    b_staticpic = true
    cpp_std = 'c++20'
    c_std = 'c20'
    backend = 'xcode'
    """)
    assert expected in content


def test_needs_exe_wrapper():
    """
    Tests needs_exe_wrapper depends on `can_run()` function instead of
    simply checking the cross_building() one.

    Issue: https://github.com/conan-io/conan/issues/19217
    """
    host = textwrap.dedent("""
    [settings]
    arch=x86_64
    build_type=Release
    compiler=apple-clang
    compiler.cppstd=gnu17
    compiler.libcxx=libc++
    compiler.version=16
    os=Macos
    [conf]
    tools.apple:sdk_path=/my/sdk/path
    tools.build.cross_building:can_run=True
    """)
    build = textwrap.dedent("""
    [settings]
    arch=armv8
    build_type=Release
    compiler=apple-clang
    compiler.cppstd=gnu17
    compiler.libcxx=libc++
    compiler.version=16
    os=Macos
    """)
    client = TestClient()
    client.save({
        "host": host,
        "build": build,
        "conanfile.py": GenConanfile("pkg", "1.0")
                        .with_settings("os", "arch", "compiler", "build_type")
                        .with_generator("MesonToolchain")
    })
    client.run("install . -pr:h host -pr:b build")
    content = client.load(MesonToolchain.cross_filename)
    assert "needs_exe_wrapper = false" in content



import os
import textwrap

import pytest

from conan.test.assets.cmake import gen_cmakelists
from conan.test.assets.genconanfile import GenConanfile
from conan.test.assets.sources import gen_function_cpp
from conan.test.utils.tools import TestClient

"""
If we have a BR with transitive requires we won't generate 'xxx-config.cmake' files for them
but neither will be included as find_dependencies()
"""


@pytest.fixture
def client():
    c = TestClient()
    zlib_conanfile = textwrap.dedent('''
    from conan import ConanFile

    class Zlib(ConanFile):
        name = "zlib"
        version = "1.2.11"

        def package_info(self):
            self.cpp_info.includedirs = []
            self.cpp_info.cxxflags = ["foo"]
    ''')
    c.save({"conanfile.py": zlib_conanfile})
    c.run("create . ")

    doxygen_conanfile = textwrap.dedent('''
    from conan import ConanFile
    from conan.tools.files import save, chdir
    import os

    class Doxygen(ConanFile):
        settings = "build_type", "os", "arch", "compiler"
        requires = "zlib/1.2.11"

        def package(self):
            with chdir(self, self.package_folder):
                save(self, "include/doxygen.h", "int foo=1;")
    ''')
    c.save({"conanfile.py": doxygen_conanfile})
    c.run("create . --name=doxygen --version=1.0")
    return c


@pytest.mark.tool("cmake")
def test_zlib_not_included(client):

    main = gen_function_cpp(name="main", includes=["doxygen.h"])
    cmake = gen_cmakelists(find_package=["doxygen"], appsources=["main.cpp"], appname="main")

    conanfile_consumer = textwrap.dedent('''
        from conan import ConanFile
        from conan.tools.cmake import CMakeDeps

        class Consumer(ConanFile):
            settings = "build_type", "os", "arch", "compiler"
            build_requires = ["doxygen/1.0"]
            generators = "CMakeToolchain"

            def generate(self):
                d = CMakeDeps(self)
                d.build_context_activated = ["doxygen"]
                d.generate()
        ''')

    client.save({"main.cpp": main, "CMakeLists.txt": cmake, "conanfile.py": conanfile_consumer},
                clean_first=True)
    client.run("install . -pr:h=default -pr:b=default")
    # The compilation works, so it finds the doxygen without transitive failures
    client.run_command("cmake . -DCMAKE_TOOLCHAIN_FILE=conan_toolchain.cmake -DCMAKE_BUILD_TYPE=Release")

    # Assert there is no zlib target
    assert "Target declared 'zlib::zlib'" not in client.out

    # Of course we find the doxygen-config.cmake
    assert os.path.exists(os.path.join(client.current_folder, "doxygen-config.cmake"))

    # The -config files for zlib are not there
    assert not os.path.exists(os.path.join(client.current_folder, "zlib-config.cmake"))


def test_error_cmakedeps_transitive_build_requires():
    """
    CMakeDeps when building an intermediate "tool_requires" that has a normal "requires"
    to other package, needs to use ``require.build`` trait instead of the more global
    "dep.is_build_context"
    We do build "protobuf" in the "build" context to make sure the whole CMakeDeps is
    working correctly
    """
    c = TestClient()
    c.run("new cmake_lib -d name=zlib -d version=0.1")
    c.run("create . -tf=")
    c.save({}, clean_first=True)
    c.run("new cmake_lib -d name=openssl -d version=0.1 -d requires=zlib/0.1")
    c.run("create . -tf=")

    # Protobuf binary is missing, to force a build below with ``--build=missing``
    c.save({}, clean_first=True)
    c.run("new cmake_exe -d name=protobuf -d version=0.1 -d requires=openssl/0.1")
    c.run("export .")

    # This conanfile used to fail, creating protobuf_mybuild*.cmake files even if for "tool" the
    # protobuf/0.1 is a regular "requires" and it is in its host context
    tool = textwrap.dedent("""
        import os
        from conan import ConanFile
        from conan.tools.cmake import CMakeDeps

        class Tool(ConanFile):
            name = "tool"
            version = "0.1"
            settings = "os", "compiler", "build_type", "arch"
            requires = "protobuf/0.1"

            def generate(self):
                deps = CMakeDeps(self)
                deps.build_context_activated = ["protobuf"]
                deps.build_context_suffix = {"protobuf": "_mybuild"}
                deps.generate()

            def build(self):
                assert os.path.exists("protobufTargets.cmake")
                assert os.path.exists("protobuf-Target-release.cmake")
        """)
    c.save({"tool/conanfile.py": tool,
            "consumer/conanfile.py": GenConanfile().with_build_requires("tool/0.1")},
           clean_first=True)
    c.run("export tool")
    c.run("install consumer --build=missing -s:b build_type=Release -s:h build_type=Debug")
    assert "tool/0.1: Created package" in c.out


def test_transitive_tool_requires_visible():
    # https://github.com/conan-io/conan/issues/16058
    # The "tool/0.1", even if visible doesn't generate any files in the "app" consumer side,
    # even if the "app" declares it in build_context_activated, because it only works for direct
    # dependencies. tool_requires(..., visible=True) only affects at the moment and version conflict
    # detection
    c = TestClient()
    c.save({"tool/conanfile.py": GenConanfile("tool", "0.1").with_package_type("application"),
            "pkgb/conanfile.py": GenConanfile("pkgb", "0.1").with_tool_requirement("tool/0.1",
                                                                                   visible=True),
            "app/conanfile.py": GenConanfile("app", "0.1").with_requires("pkgb/0.1")
                                                          .with_settings("build_type")
                                                          .with_generator("CMakeDeps")})

    c.run("create tool")
    c.run("create pkgb")
    c.run("install app")
    cmake = c.load("app/pkgb-release-data.cmake")
    # pkgb doesn't deepnd on tool, because it is a tool-require and not build-context activated
    assert "list(APPEND pkgb_FIND_DEPENDENCY_NAMES )" in cmake



from conan.api.model import PkgReference


def test_package_reference():
    r = PkgReference.loads("pkg/0.1:pkgid1")
    assert r.ref.name == "pkg"
    assert r.ref.version == "0.1"
    assert r.ref.user is None
    assert r.ref.channel is None
    assert r.ref.revision is None
    assert r.package_id == "pkgid1"
    assert r.revision is None
    assert str(r) == "pkg/0.1:pkgid1"
    assert repr(r) == "pkg/0.1:pkgid1"

    r = PkgReference.loads("pkg/0.1@user:pkgid1")
    assert r.ref.name == "pkg"
    assert r.ref.version == "0.1"
    assert r.ref.user == "user"
    assert r.ref.channel is None
    assert r.ref.revision is None
    assert r.package_id == "pkgid1"
    assert r.revision is None
    assert str(r) == "pkg/0.1@user:pkgid1"
    assert repr(r) == "pkg/0.1@user:pkgid1"



import json

from conan.test.assets.genconanfile import GenConanfile
from conan.test.utils.tools import TestClient


def test_build_cmd_deploy_generators():
    """
    Test that "conan build --deployer/--generators" args work
    """
    c = TestClient()
    c.save({"dep/conanfile.py": GenConanfile("dep", "1.0"),
            "pkg/conanfile.py": GenConanfile("pkg", "1.0").with_settings("build_type")
                                                          .with_requires("dep/1.0")})
    c.run("create dep")
    c.run("build pkg --deployer=full_deploy --deployer-folder=./myfolder -g=CMakeDeps")
    cmake = c.load("pkg/dep-release-data.cmake")
    current_folder = c.current_folder.replace("\\", "/")
    path = f'{current_folder}/myfolder/full_deploy/host/dep/1.0'
    assert f'set(dep_PACKAGE_FOLDER_RELEASE "{path}")' in cmake


def test_build_output_json():
    """
    The build command should have the --format=json option.
    """
    _OUTPUT_FILE = "output.json"

    client = TestClient()
    conanfile = GenConanfile()
    client.save({"conanfile.py": conanfile})
    client.run("build --format=json", redirect_stdout=_OUTPUT_FILE)
    output = json.loads(client.load(_OUTPUT_FILE))

    assert "graph" in output
    assert "nodes" in output["graph"]



import textwrap

import pytest

from conan.test.assets.genconanfile import GenConanfile
from conan.test.utils.tools import TestClient


@pytest.fixture
def client():
    conanfile = textwrap.dedent("""
       from conan import ConanFile
       class Pkg(ConanFile):
           def generate(self):
               for var in (1, 2):
                   v = self.buildenv.vars(self).get("MyVar{}".format(var))
                   self.output.info("MyVar{}={}!!".format(var, v))
       """)
    profile1 = textwrap.dedent("""
      [buildenv]
      MyVar1=MyValue1_1
      MyVar2=MyValue2_1
      """)
    client = TestClient()
    client.save({"conanfile.py": conanfile,
                 "profile1": profile1})
    return client


def test_buildenv_profile_cli(client):
    profile2 = textwrap.dedent("""
        [buildenv]
        MyVar1=MyValue1_2
        MyVar2+=MyValue2_2
        """)
    client.save({"profile2": profile2})

    client.run("install . -pr=profile1 -pr=profile2")
    assert "conanfile.py: MyVar1=MyValue1_2!!" in client.out
    assert "conanfile.py: MyVar2=MyValue2_1 MyValue2_2" in client.out


def test_buildenv_profile_include(client):
    profile2 = textwrap.dedent("""
        include(profile1)
        [buildenv]
        MyVar1=MyValue1_2
        MyVar2+=MyValue2_2
        """)
    client.save({"profile2": profile2})

    client.run("install . -pr=profile2")
    assert "conanfile.py: MyVar1=MyValue1_2!!" in client.out
    assert "conanfile.py: MyVar2=MyValue2_1 MyValue2_2" in client.out


def test_buildenv_package_patterns():
    client = TestClient()
    conanfile = GenConanfile()
    generate = """
    def generate(self):
        value = self.buildenv.vars(self).get("my_env_var") or "None"
        self.output.warning("{} ENV:{}".format(self.ref.name, value))
"""
    client.save({"dep/conanfile.py": str(conanfile) + generate,
                 "pkg/conanfile.py": str(conanfile.with_requirement("dep/0.1", visible=False)) + generate,
                 "consumer/conanfile.py": str(conanfile.with_requires("pkg/0.1")
                .with_settings("os", "build_type", "arch")) + generate})

    client.run("export dep --name=dep --version=0.1")
    client.run("export pkg --name=pkg --version=0.1")

    # This pattern applies to no package
    profile = """
            include(default)
            [buildenv]
            invented/*:my_env_var=Foo
            """
    client.save({"profile": profile})
    client.run("install consumer --build='*' --profile profile")
    assert "WARN: dep ENV:None" in client.out
    assert "WARN: pkg ENV:None" in client.out
    assert "WARN: None ENV:None" in client.out

    # This patterns applies to dep
    profile = """
                include(default)
                [buildenv]
                dep/*:my_env_var=Foo
                """
    client.save({"profile": profile})
    client.run("install consumer --build='*' --profile profile")
    assert "WARN: dep ENV:Foo" in client.out
    assert "WARN: pkg ENV:None" in client.out
    assert "WARN: None ENV:None" in client.out

    profile = """
                    include(default)
                    [buildenv]
                    dep/0.1:my_env_var=Foo
                    """
    client.save({"profile": profile})
    client.run("install consumer --build='*' --profile profile")
    assert "WARN: dep ENV:Foo" in client.out
    assert "WARN: pkg ENV:None" in client.out
    assert "WARN: None ENV:None" in client.out

    # The global pattern applies to all
    profile = """
                    include(default)
                    [buildenv]
                    dep/*:my_env_var=Foo
                    pkg/*:my_env_var=Foo
                    my_env_var=Var
                    """
    client.save({"profile": profile})
    client.run("install consumer --build='*' --profile profile")
    assert "WARN: dep ENV:Var" in client.out
    assert "WARN: pkg ENV:Var" in client.out
    assert "WARN: None ENV:Var" in client.out

    # "&" pattern for the consumer
    profile = """
                        include(default)
                        [buildenv]
                        dep/*:my_env_var=Foo
                        pkg/*:my_env_var=Foo2
                        &:my_env_var=Var
                        """
    client.save({"profile": profile})
    client.run("install consumer --build='*' --profile profile")
    assert "WARN: dep ENV:Foo" in client.out
    assert "WARN: pkg ENV:Foo2" in client.out
    assert "WARN: None ENV:Var" in client.out


def test_buildenv_error_unset():
    # https://github.com/conan-io/conan/issues/19285#issuecomment-3569891282
    c = TestClient()
    profile = textwrap.dedent("""
        [buildenv]
        CLASSPATH=!
        OTHERPATH=
        """)
    c.save({"conanfile.txt": "",
            "profile": profile})

    c.run("install . -pr=profile -s:a os=Linux")
    env = c.load("conanbuildenv.sh")
    assert "unset CLASSPATH" in env
    assert 'export OTHERPATH=""' in env



import platform
import textwrap

import pytest

from conan.test.utils.tools import TestClient


@pytest.mark.tool("pkg_config")
@pytest.mark.tool("autotools")
@pytest.mark.skipif(platform.system() == "Windows", reason="Needs pkg-config")
def test_pkgconfigdeps_and_autotools():
    """
    This test aims to show how to use PkgConfigDeps and AutotoolsToolchain.

    In this case, the test_package is using PkgConfigDeps and AutotoolsToolchain to use
    the created pkg/1.0 package. It's important to make a full compilation to ensure that
    the test main.cpp is using correctly the flags passed via pkg.pc file.

    Issue related: https://github.com/conan-io/conan/issues/11867
    """
    client = TestClient(path_with_spaces=False)
    conanfile_pkg = textwrap.dedent("""
    from conan import ConanFile
    from conan.tools.gnu import Autotools
    from conan.tools.layout import basic_layout
    from conan.tools.apple import fix_apple_shared_install_name


    class PkgConan(ConanFile):
        name = "pkg"
        version = "1.0"
        settings = "os", "compiler", "build_type", "arch"
        options = {"shared": [True, False], "fPIC": [True, False]}
        default_options = {"shared": False, "fPIC": True}
        exports_sources = "configure.ac", "Makefile.am", "src/*"
        generators = "AutotoolsToolchain"

        def layout(self):
            basic_layout(self)

        def build(self):
            autotools = Autotools(self)
            autotools.autoreconf()
            autotools.configure()
            autotools.make()

        def package(self):
            autotools = Autotools(self)
            autotools.install()
            fix_apple_shared_install_name(self)

        def package_info(self):
            self.cpp_info.libs = ["pkg"]
            # Add non-existing frameworkdirs to check that it's not failing because of this
            self.cpp_info.frameworkdirs = ["/my/framework/file1", "/my/framework/file2"]
    """)
    conanfile_test = textwrap.dedent("""
    import os
    from conan import ConanFile
    from conan.tools.gnu import Autotools
    from conan.tools.layout import basic_layout
    from conan.tools.build import cross_building
    class PkgTestConan(ConanFile):
        settings = "os", "compiler", "build_type", "arch"
        generators = "PkgConfigDeps", "AutotoolsToolchain"

        def requirements(self):
            self.requires(self.tested_reference_str)

        def build(self):
            autotools = Autotools(self)
            autotools.autoreconf()
            autotools.configure()
            autotools.make()

        def layout(self):
            basic_layout(self)

        def test(self):
            if not cross_building(self):
                cmd = os.path.join(self.cpp.build.bindirs[0], "main")
                self.run(cmd, env="conanrun")
    """)
    configure_test = textwrap.dedent("""
    AC_INIT([main], [1.0], [])
    AM_INIT_AUTOMAKE([-Wall -Werror foreign])
    AC_PROG_CXX
    PKG_PROG_PKG_CONFIG
    PKG_CHECK_MODULES([pkg], [pkg >= 1.0])
    AC_CONFIG_FILES([Makefile])
    AC_OUTPUT
    """)
    makefile_test = textwrap.dedent("""
    bin_PROGRAMS = main
    main_SOURCES = main.cpp
    AM_CXXFLAGS = $(pkg_CFLAGS)
    main_LDADD = $(pkg_LIBS)
    """)
    client.run("new autotools_lib -d name=pkg -d version=1.0")
    client.save({"conanfile.py": conanfile_pkg,
                 "test_package/conanfile.py": conanfile_test,
                 "test_package/configure.ac": configure_test,
                 "test_package/Makefile.am": makefile_test,
                 })
    # client.run("new autotools_lib -d name=pkg -d version=1.0")
    client.run("create .")
    if platform.system() == "Darwin":
        # Checking that frameworkdirs appear all together instead of "-F /whatever/f1"
        # Issue: https://github.com/conan-io/conan/issues/11867
        assert '-F/my/framework/file1' in client.out
        assert '-F/my/framework/file2' in client.out




from conan.api.output import ConanOutput
from conan.internal.model.manifest import FileTreeManifest
from conan.test.utils.mocks import RedirectedTestOutput
from conan.test.utils.tools import redirect_output


class TestReportCopiedFiles:

    def test_output_string(self):
        manifest = FileTreeManifest(0,
                                    file_sums={'/abs/path/to/file.pdf': "",
                                               '../rel/path/to/file2.pdf': "",
                                               '../rel/path/to/file3.pdf': "",
                                               '../rel/path/to/file4.pdf': "",
                                               '../rel/path/to/file5.pdf': "",
                                               '../rel/path/to/file6.pdf': "",
                                               '../rel/path/to/file7.pdf': "",
                                               '/without/ext/no_ext1': "",
                                               'no_ext2': "",
                                               'a/other.txt': ""})
        output = RedirectedTestOutput()
        with redirect_output(output):
            manifest.report_summary(ConanOutput())
            lines = sorted(str(output).splitlines())
            assert "Copied 7 '.pdf' files" == lines[2]
            assert "Copied 2 files: no_ext1, no_ext2" == lines[1]
            assert "Copied 1 '.txt' file: other.txt" == lines[0]



import json
import textwrap

import pytest

from conan.test.utils.tools import TestClient


@pytest.fixture
def client():
    client = TestClient()
    conanfile = textwrap.dedent("""
        from conan import ConanFile

        class Pkg(ConanFile):
            def package_id(self):
                self.info.conf.define("user.myconf:myitem", self.conf.get("user.myconf:myitem"))
        """)
    client.save({"conanfile.py": conanfile})
    return client


def test_package_id(client):
    profile1 = textwrap.dedent("""\
        [conf]
        user.myconf:myitem=1""")
    profile2 = textwrap.dedent("""\
        [conf]
        user.myconf:myitem=2""")
    client.save({"profile1": profile1,
                 "profile2": profile2})
    client.run("create . --name=pkg --version=0.1 -pr=profile1")
    client.assert_listed_binary({"pkg/0.1": ("7501c16e6c5c93e534dd760829859340e2dc73bc", "Build")})
    client.run("create . --name=pkg --version=0.1 -pr=profile2")
    client.assert_listed_binary({"pkg/0.1": ("4eb2bd276f75d2df8fa0f4b58bd86014b7f51693", "Build")})


def test_json_output(client):
    client.run("create . --name=pkg --version=0.1 -c user.myconf:myitem=1 --format=json")
    graph = json.loads(client.stdout)
    assert graph["graph"]["nodes"]["1"]["info"]["conf"] == {'user.myconf:myitem': 1}
